#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#

"""Security Group Rule action implementations"""

import argparse
import logging

from osc_lib.cli import parseractions
from osc_lib import exceptions
from osc_lib import utils

from openstackclient.i18n import _
from openstackclient.identity import common as identity_common
from openstackclient.network import common
from openstackclient.network import utils as network_utils

LOG = logging.getLogger(__name__)


def _format_security_group_rule_show(obj):
    data = network_utils.transform_compute_security_group_rule(obj)
    return zip(*sorted(data.items()))


def _format_network_port_range(rule):
    # Display port range or ICMP type and code. For example:
    # - ICMP type: 'type=3'
    # - ICMP type and code: 'type=3:code=0'
    # - ICMP code: Not supported
    # - Matching port range: '443:443'
    # - Different port range: '22:24'
    # - Single port: '80:80'
    # - No port range: ''
    port_range = ''
    if _is_icmp_protocol(rule['protocol']):
        if rule['port_range_min']:
            port_range += 'type=' + str(rule['port_range_min'])
        if rule['port_range_max']:
            port_range += ':code=' + str(rule['port_range_max'])
    elif rule['port_range_min'] or rule['port_range_max']:
        port_range_min = str(rule['port_range_min'])
        port_range_max = str(rule['port_range_max'])
        if rule['port_range_min'] is None:
            port_range_min = port_range_max
        if rule['port_range_max'] is None:
            port_range_max = port_range_min
        port_range = port_range_min + ':' + port_range_max
    return port_range


def _format_remote_ip_prefix(rule):
    remote_ip_prefix = rule['remote_ip_prefix']
    if remote_ip_prefix is None:
        ethertype = rule['ether_type']
        if ethertype == 'IPv4':
            remote_ip_prefix = '0.0.0.0/0'
        elif ethertype == 'IPv6':
            remote_ip_prefix = '::/0'
    return remote_ip_prefix


def _get_columns(item):
    column_map = {}
    hidden_columns = ['location', 'tenant_id']
    return utils.get_osc_show_columns_for_sdk_resource(
        item,
        column_map,
        hidden_columns
    )


def _convert_to_lowercase(string):
    return string.lower()


def _convert_ipvx_case(string):
    if string.lower() == 'ipv4':
        return 'IPv4'
    if string.lower() == 'ipv6':
        return 'IPv6'
    return string


def _is_icmp_protocol(protocol):
    # NOTE(rtheis): Neutron has deprecated protocol icmpv6.
    # However, while the OSC CLI doesn't document the protocol,
    # the code must still handle it. In addition, handle both
    # protocol names and numbers.
    if protocol in ['icmp', 'icmpv6', 'ipv6-icmp', '1', '58']:
        return True
    else:
        return False


# TODO(abhiraut): Use the SDK resource mapped attribute names once the
# OSC minimum requirements include SDK 1.0.
class CreateSecurityGroupRule(common.NetworkAndComputeShowOne,
                              common.NeutronCommandWithExtraArgs):
    _description = _("Create a new security group rule")

    def update_parser_common(self, parser):
        parser.add_argument(
            'group',
            metavar='<group>',
            help=_("Create rule in this security group (name or ID)")
        )
        remote_group = parser.add_mutually_exclusive_group()
        remote_group.add_argument(
            "--remote-ip",
            metavar="<ip-address>",
            help=_("Remote IP address block (may use CIDR notation; "
                   "default for IPv4 rule: 0.0.0.0/0, "
                   "default for IPv6 rule: ::/0)"),
        )
        remote_group.add_argument(
            "--remote-group",
            metavar="<group>",
            help=_("Remote security group (name or ID)"),
        )
        if self.is_neutron:
            remote_group.add_argument(
                "--remote-address-group",
                metavar="<group>",
                help=_("Remote address group (name or ID)"),
            )

        # NOTE(efried): The --dst-port, --protocol, and --proto options exist
        # for both nova-network and neutron, but differ slightly. For the sake
        # of the docs build, which has to account for both variants, but only
        # add each to the parser once, they are handled here rather than in the
        # _network- or _compute-specific methods below.

        # --dst-port has a default for nova-net only
        if self.is_nova_network:
            dst_port_default = dict(default=(0, 0))
        else:
            dst_port_default = {}
        parser.add_argument(
            '--dst-port',
            metavar='<port-range>',
            action=parseractions.RangeAction,
            help=_("Destination port, may be a single port or a starting and "
                   "ending port range: 137:139. Required for IP protocols TCP "
                   "and UDP. Ignored for ICMP IP protocols."),
            **dst_port_default
        )

        # NOTE(rtheis): Support either protocol option name for now.
        # However, consider deprecating and then removing --proto in
        # a future release.
        protocol_group = parser.add_mutually_exclusive_group()
        # --proto[col] has choices for nova-network only
        if self.is_nova_network:
            proto_choices = dict(choices=['icmp', 'tcp', 'udp'])
        else:
            proto_choices = {}
        protocol_help_compute = _("IP protocol (icmp, tcp, udp; default: tcp)")
        protocol_help_network = _(
            "IP protocol (ah, dccp, egp, esp, gre, icmp, igmp, ipv6-encap, "
            "ipv6-frag, ipv6-icmp, ipv6-nonxt, ipv6-opts, ipv6-route, ospf, "
            "pgm, rsvp, sctp, tcp, udp, udplite, vrrp and integer "
            "representations [0-255] or any; default: any (all protocols))")
        if self.is_nova_network:
            protocol_help = protocol_help_compute
        elif self.is_neutron:
            protocol_help = protocol_help_network
        else:
            # Docs build: compose help for both nova-network and neutron
            protocol_help = self.split_help(
                protocol_help_network, protocol_help_compute)

        protocol_group.add_argument(
            '--protocol',
            metavar='<protocol>',
            type=_convert_to_lowercase,
            help=protocol_help,
            **proto_choices
        )
        if not self.is_docs_build:
            protocol_group.add_argument(
                '--proto',
                metavar='<proto>',
                type=_convert_to_lowercase,
                help=argparse.SUPPRESS,
                **proto_choices
            )

        return parser

    def update_parser_network(self, parser):
        parser.add_argument(
            '--description',
            metavar='<description>',
            help=self.enhance_help_neutron(
                _("Set security group rule description"))
        )
        parser.add_argument(
            '--icmp-type',
            metavar='<icmp-type>',
            type=int,
            help=self.enhance_help_neutron(
                _("ICMP type for ICMP IP protocols"))
        )
        parser.add_argument(
            '--icmp-code',
            metavar='<icmp-code>',
            type=int,
            help=self.enhance_help_neutron(
                _("ICMP code for ICMP IP protocols"))
        )
        direction_group = parser.add_mutually_exclusive_group()
        direction_group.add_argument(
            '--ingress',
            action='store_true',
            help=self.enhance_help_neutron(
                _("Rule applies to incoming network traffic (default)"))
        )
        direction_group.add_argument(
            '--egress',
            action='store_true',
            help=self.enhance_help_neutron(
                _("Rule applies to outgoing network traffic"))
        )
        parser.add_argument(
            '--ethertype',
            metavar='<ethertype>',
            choices=['IPv4', 'IPv6'],
            type=_convert_ipvx_case,
            help=self.enhance_help_neutron(
                _("Ethertype of network traffic "
                  "(IPv4, IPv6; default: based on IP protocol)"))
        )
        parser.add_argument(
            '--project',
            metavar='<project>',
            help=self.enhance_help_neutron(_("Owner's project (name or ID)"))
        )
        identity_common.add_project_domain_option_to_parser(
            parser, enhance_help=self.enhance_help_neutron)
        return parser

    def _get_protocol(self, parsed_args, default_protocol='any'):
        protocol = default_protocol
        if parsed_args.protocol is not None:
            protocol = parsed_args.protocol
        if parsed_args.proto is not None:
            protocol = parsed_args.proto
        if protocol == 'any':
            protocol = None
        return protocol

    def _get_ethertype(self, parsed_args, protocol):
        ethertype = 'IPv4'
        if parsed_args.ethertype is not None:
            ethertype = parsed_args.ethertype
        elif self._is_ipv6_protocol(protocol):
            ethertype = 'IPv6'
        return ethertype

    def _is_ipv6_protocol(self, protocol):
        # NOTE(rtheis): Neutron has deprecated protocol icmpv6.
        # However, while the OSC CLI doesn't document the protocol,
        # the code must still handle it. In addition, handle both
        # protocol names and numbers.
        if (protocol is not None and protocol.startswith('ipv6-') or
                protocol in ['icmpv6', '41', '43', '44', '58', '59', '60']):
            return True
        else:
            return False

    def take_action_network(self, client, parsed_args):
        # Get the security group ID to hold the rule.
        security_group_id = client.find_security_group(
            parsed_args.group,
            ignore_missing=False
        ).id

        # Build the create attributes.
        attrs = {}
        attrs['protocol'] = self._get_protocol(parsed_args)

        if parsed_args.description is not None:
            attrs['description'] = parsed_args.description

        # NOTE(rtheis): A direction must be specified and ingress
        # is the default.
        if parsed_args.ingress or not parsed_args.egress:
            attrs['direction'] = 'ingress'
        if parsed_args.egress:
            attrs['direction'] = 'egress'

        # NOTE(rtheis): Use ethertype specified else default based
        # on IP protocol.
        attrs['ethertype'] = self._get_ethertype(parsed_args,
                                                 attrs['protocol'])

        # NOTE(rtheis): Validate the port range and ICMP type and code.
        # It would be ideal if argparse could do this.
        if parsed_args.dst_port and (parsed_args.icmp_type or
                                     parsed_args.icmp_code):
            msg = _('Argument --dst-port not allowed with arguments '
                    '--icmp-type and --icmp-code')
            raise exceptions.CommandError(msg)
        if parsed_args.icmp_type is None and parsed_args.icmp_code is not None:
            msg = _('Argument --icmp-type required with argument --icmp-code')
            raise exceptions.CommandError(msg)
        is_icmp_protocol = _is_icmp_protocol(attrs['protocol'])
        if not is_icmp_protocol and (parsed_args.icmp_type or
                                     parsed_args.icmp_code):
            msg = _('ICMP IP protocol required with arguments '
                    '--icmp-type and --icmp-code')
            raise exceptions.CommandError(msg)
        # NOTE(rtheis): For backwards compatibility, continue ignoring
        # the destination port range when an ICMP IP protocol is specified.
        if parsed_args.dst_port and not is_icmp_protocol:
            attrs['port_range_min'] = parsed_args.dst_port[0]
            attrs['port_range_max'] = parsed_args.dst_port[1]
        if parsed_args.icmp_type is not None and parsed_args.icmp_type >= 0:
            attrs['port_range_min'] = parsed_args.icmp_type
        if parsed_args.icmp_code is not None and parsed_args.icmp_code >= 0:
            attrs['port_range_max'] = parsed_args.icmp_code

        if parsed_args.remote_group is not None:
            attrs['remote_group_id'] = client.find_security_group(
                parsed_args.remote_group,
                ignore_missing=False
            ).id
        elif parsed_args.remote_address_group is not None:
            attrs['remote_address_group_id'] = client.find_address_group(
                parsed_args.remote_address_group,
                ignore_missing=False
            ).id
        elif parsed_args.remote_ip is not None:
            attrs['remote_ip_prefix'] = parsed_args.remote_ip
        elif attrs['ethertype'] == 'IPv4':
            attrs['remote_ip_prefix'] = '0.0.0.0/0'
        elif attrs['ethertype'] == 'IPv6':
            attrs['remote_ip_prefix'] = '::/0'
        attrs['security_group_id'] = security_group_id
        if parsed_args.project is not None:
            identity_client = self.app.client_manager.identity
            project_id = identity_common.find_project(
                identity_client,
                parsed_args.project,
                parsed_args.project_domain,
            ).id
            attrs['project_id'] = project_id

        attrs.update(
            self._parse_extra_properties(parsed_args.extra_properties))

        # Create and show the security group rule.
        obj = client.create_security_group_rule(**attrs)
        display_columns, columns = _get_columns(obj)
        data = utils.get_item_properties(obj, columns)
        return (display_columns, data)

    def take_action_compute(self, client, parsed_args):
        group = client.api.security_group_find(parsed_args.group)
        protocol = self._get_protocol(parsed_args, default_protocol='tcp')
        if protocol == 'icmp':
            from_port, to_port = -1, -1
        else:
            from_port, to_port = parsed_args.dst_port

        remote_ip = None
        if parsed_args.remote_group is not None:
            parsed_args.remote_group = client.api.security_group_find(
                parsed_args.remote_group,
            )['id']
        if parsed_args.remote_ip is not None:
            remote_ip = parsed_args.remote_ip
        else:
            remote_ip = '0.0.0.0/0'

        obj = client.api.security_group_rule_create(
            security_group_id=group['id'],
            ip_protocol=protocol,
            from_port=from_port,
            to_port=to_port,
            remote_ip=remote_ip,
            remote_group=parsed_args.remote_group,
        )
        return _format_security_group_rule_show(obj)


class DeleteSecurityGroupRule(common.NetworkAndComputeDelete):
    _description = _("Delete security group rule(s)")

    # Used by base class to find resources in parsed_args.
    resource = 'rule'
    r = None

    def update_parser_common(self, parser):
        parser.add_argument(
            'rule',
            metavar='<rule>',
            nargs="+",
            help=_("Security group rule(s) to delete (ID only)")
        )
        return parser

    def take_action_network(self, client, parsed_args):
        obj = client.find_security_group_rule(
            self.r, ignore_missing=False)
        client.delete_security_group_rule(obj)

    def take_action_compute(self, client, parsed_args):
        client.api.security_group_rule_delete(self.r)


class ListSecurityGroupRule(common.NetworkAndComputeLister):
    _description = _("List security group rules")

    def _format_network_security_group_rule(self, rule):
        """Transform the SDK SecurityGroupRule object to a dict

        The SDK object gets in the way of reformatting columns...
        Create port_range column from port_range_min and port_range_max
        """
        rule = rule.to_dict()
        rule['port_range'] = _format_network_port_range(rule)
        rule['remote_ip_prefix'] = _format_remote_ip_prefix(rule)
        return rule

    def update_parser_common(self, parser):
        parser.add_argument(
            'group',
            metavar='<group>',
            nargs='?',
            help=_("List all rules in this security group (name or ID)")
        )
        return parser

    def update_parser_network(self, parser):
        if not self.is_docs_build:
            # Accept but hide the argument for consistency with compute.
            # Network will always return all projects for an admin.
            parser.add_argument(
                '--all-projects',
                action='store_true',
                default=False,
                help=argparse.SUPPRESS
            )

        parser.add_argument(
            '--protocol',
            metavar='<protocol>',
            type=_convert_to_lowercase,
            help=self.enhance_help_neutron(
                _("List rules by the IP protocol (ah, dhcp, egp, esp, gre, "
                  "icmp, igmp, ipv6-encap, ipv6-frag, ipv6-icmp, ipv6-nonxt, "
                  "ipv6-opts, ipv6-route, ospf, pgm, rsvp, sctp, tcp, udp, "
                  "udplite, vrrp and integer representations [0-255] or any; "
                  "default: any (all protocols))"))
        )
        parser.add_argument(
            '--ethertype',
            metavar='<ethertype>',
            type=_convert_to_lowercase,
            help=self.enhance_help_neutron(
                _("List rules by the Ethertype (IPv4 or IPv6)"))
        )
        direction_group = parser.add_mutually_exclusive_group()
        direction_group.add_argument(
            '--ingress',
            action='store_true',
            help=self.enhance_help_neutron(
                _("List rules applied to incoming network traffic"))
        )
        direction_group.add_argument(
            '--egress',
            action='store_true',
            help=self.enhance_help_neutron(
                _("List rules applied to outgoing network traffic"))
        )
        parser.add_argument(
            '--long',
            action='store_true',
            default=False,
            help=self.enhance_help_neutron(
                _("**Deprecated** This argument is no longer needed"))
        )
        return parser

    def update_parser_compute(self, parser):
        parser.add_argument(
            '--all-projects',
            action='store_true',
            default=False,
            help=self.enhance_help_nova_network(
                _("Display information from all projects (admin only)"))
        )
        if not self.is_docs_build:
            # Accept but hide the argument for consistency with network.
            # There are no additional fields to display at this time.
            parser.add_argument(
                '--long',
                action='store_false',
                default=False,
                help=argparse.SUPPRESS
            )
        return parser

    def _get_column_headers(self, parsed_args):
        column_headers = (
            'ID',
            'IP Protocol',
            'Ethertype',
            'IP Range',
            'Port Range',
            'Direction',
            'Remote Security Group',
        )
        if self.is_neutron:
            column_headers = column_headers + ('Remote Address Group',)
        if parsed_args.group is None:
            column_headers = column_headers + ('Security Group',)
        return column_headers

    def take_action_network(self, client, parsed_args):
        if parsed_args.long:
            self.log.warning(_(
                "The --long option has been deprecated and is no longer needed"
            ))

        column_headers = self._get_column_headers(parsed_args)
        columns = (
            'id',
            'protocol',
            'ether_type',
            'remote_ip_prefix',
            'port_range',
            'direction',
            'remote_group_id',
            'remote_address_group_id',
        )

        # Get the security group rules using the requested query.
        query = {}
        if parsed_args.group is not None:
            # NOTE(rtheis): Unfortunately, the security group resource
            # does not contain security group rules resources. So use
            # the security group ID in a query to get the resources.
            security_group_id = client.find_security_group(
                parsed_args.group,
                ignore_missing=False
            ).id
            query = {'security_group_id': security_group_id}
        else:
            columns = columns + ('security_group_id',)

        if parsed_args.ingress:
            query['direction'] = 'ingress'
        if parsed_args.egress:
            query['direction'] = 'egress'
        if parsed_args.protocol is not None:
            query['protocol'] = parsed_args.protocol

        rules = [
            self._format_network_security_group_rule(r)
            for r in client.security_group_rules(**query)
        ]

        return (column_headers,
                (utils.get_dict_properties(
                    s, columns,
                ) for s in rules))

    def take_action_compute(self, client, parsed_args):
        column_headers = self._get_column_headers(parsed_args)
        columns = (
            "ID",
            "IP Protocol",
            "Ethertype",
            "IP Range",
            "Port Range",
            "Remote Security Group",
        )

        rules_to_list = []
        if parsed_args.group is not None:
            group = client.api.security_group_find(
                parsed_args.group,
            )
            rules_to_list = group['rules']
        else:
            columns = columns + ('parent_group_id',)
            search = {'all_tenants': parsed_args.all_projects}
            for group in client.api.security_group_list(search_opts=search):
                rules_to_list.extend(group['rules'])

        # NOTE(rtheis): Turn the raw rules into resources.
        rules = []
        for rule in rules_to_list:
            rules.append(
                network_utils.transform_compute_security_group_rule(rule),
            )
            # rules.append(compute_secgroup_rules.SecurityGroupRule(
            #     client.security_group_rules,
            #     network_utils.transform_compute_security_group_rule(rule),
            # ))

        return (column_headers,
                (utils.get_dict_properties(
                    s, columns,
                ) for s in rules))


class ShowSecurityGroupRule(common.NetworkAndComputeShowOne):
    _description = _("Display security group rule details")

    def update_parser_common(self, parser):
        parser.add_argument(
            'rule',
            metavar="<rule>",
            help=_("Security group rule to display (ID only)")
        )
        return parser

    def take_action_network(self, client, parsed_args):
        obj = client.find_security_group_rule(parsed_args.rule,
                                              ignore_missing=False)
        # necessary for old rules that have None in this field
        if not obj['remote_ip_prefix']:
            obj['remote_ip_prefix'] = _format_remote_ip_prefix(obj)
        display_columns, columns = _get_columns(obj)
        data = utils.get_item_properties(obj, columns)
        return (display_columns, data)

    def take_action_compute(self, client, parsed_args):
        # NOTE(rtheis): Unfortunately, compute does not have an API
        # to get or list security group rules so parse through the
        # security groups to find all accessible rules in search of
        # the requested rule.
        obj = None
        security_group_rules = []
        for security_group in client.api.security_group_list():
            security_group_rules.extend(security_group['rules'])
        for security_group_rule in security_group_rules:
            if parsed_args.rule == str(security_group_rule.get('id')):
                obj = security_group_rule
                break

        if obj is None:
            msg = _("Could not find security group rule "
                    "with ID '%s'") % parsed_args.rule
            raise exceptions.CommandError(msg)

        # NOTE(rtheis): Format security group rule
        return _format_security_group_rule_show(obj)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#

"""Identity v3 Trust action implementations"""

import datetime
import logging

from keystoneclient import exceptions as identity_exc
from osc_lib.command import command
from osc_lib import exceptions
from osc_lib import utils

from openstackclient.i18n import _
from openstackclient.identity import common


LOG = logging.getLogger(__name__)


class CreateTrust(command.ShowOne):
    _description = _("Create new trust")

    def get_parser(self, prog_name):
        parser = super(CreateTrust, self).get_parser(prog_name)
        parser.add_argument(
            'trustor',
            metavar='<trustor-user>',
            help=_('User that is delegating authorization (name or ID)'),
        )
        parser.add_argument(
            'trustee',
            metavar='<trustee-user>',
            help=_('User that is assuming authorization (name or ID)'),
        )
        parser.add_argument(
            '--project',
            metavar='<project>',
            required=True,
            help=_('Project being delegated (name or ID) (required)'),
        )
        parser.add_argument(
            '--role',
            metavar='<role>',
            action='append',
            default=[],
            help=_('Roles to authorize (name or ID) '
                   '(repeat option to set multiple values, required)'),
            required=True
        )
        parser.add_argument(
            '--impersonate',
            dest='impersonate',
            action='store_true',
            default=False,
            help=_('Tokens generated from the trust will represent <trustor>'
                   ' (defaults to False)'),
        )
        parser.add_argument(
            '--expiration',
            metavar='<expiration>',
            help=_('Sets an expiration date for the trust'
                   ' (format of YYYY-mm-ddTHH:MM:SS)'),
        )
        common.add_project_domain_option_to_parser(parser)
        parser.add_argument(
            '--trustor-domain',
            metavar='<trustor-domain>',
            help=_('Domain that contains <trustor> (name or ID)'),
        )
        parser.add_argument(
            '--trustee-domain',
            metavar='<trustee-domain>',
            help=_('Domain that contains <trustee> (name or ID)'),
        )
        return parser

    def take_action(self, parsed_args):
        identity_client = self.app.client_manager.identity

        # NOTE(stevemar): Find the two users, project and roles that
        # are necessary for making a trust usable, the API dictates that
        # trustee, project and role are optional, but that makes the trust
        # pointless, and trusts are immutable, so let's enforce it at the
        # client level.
        trustor_id = common.find_user(identity_client,
                                      parsed_args.trustor,
                                      parsed_args.trustor_domain).id
        trustee_id = common.find_user(identity_client,
                                      parsed_args.trustee,
                                      parsed_args.trustee_domain).id
        project_id = common.find_project(identity_client,
                                         parsed_args.project,
                                         parsed_args.project_domain).id

        role_ids = []
        for role in parsed_args.role:
            try:
                role_id = utils.find_resource(
                    identity_client.roles,
                    role,
                ).id
            except identity_exc.Forbidden:
                role_id = role
            role_ids.append(role_id)

        expires_at = None
        if parsed_args.expiration:
            expires_at = datetime.datetime.strptime(parsed_args.expiration,
                                                    '%Y-%m-%dT%H:%M:%S')

        trust = identity_client.trusts.create(
            trustee_id, trustor_id,
            impersonation=parsed_args.impersonate,
            project=project_id,
            role_ids=role_ids,
            expires_at=expires_at,
        )

        trust._info.pop('roles_links', None)
        trust._info.pop('links', None)

        # Format roles into something sensible
        roles = trust._info.pop('roles')
        msg = ' '.join(r['name'] for r in roles)
        trust._info['roles'] = msg

        return zip(*sorted(trust._info.items()))


class DeleteTrust(command.Command):
    _description = _("Delete trust(s)")

    def get_parser(self, prog_name):
        parser = super(DeleteTrust, self).get_parser(prog_name)
        parser.add_argument(
            'trust',
            metavar='<trust>',
            nargs="+",
            help=_('Trust(s) to delete'),
        )
        return parser

    def take_action(self, parsed_args):
        identity_client = self.app.client_manager.identity

        errors = 0
        for trust in parsed_args.trust:
            try:
                trust_obj = utils.find_resource(identity_client.trusts,
                                                trust)
                identity_client.trusts.delete(trust_obj.id)
            except Exception as e:
                errors += 1
                LOG.error(_("Failed to delete trust with "
                          "name or ID '%(trust)s': %(e)s"),
                          {'trust': trust, 'e': e})

        if errors > 0:
            total = len(parsed_args.trust)
            msg = (_("%(errors)s of %(total)s trusts failed "
                   "to delete.") % {'errors': errors, 'total': total})
            raise exceptions.CommandError(msg)


class ListTrust(command.Lister):
    _description = _("List trusts")

    def get_parser(self, prog_name):
        parser = super().get_parser(prog_name)
        parser.add_argument(
            '--trustor',
            metavar='<trustor-user>',
            help=_('Trustor user to filter (name or ID)'),
        )
        parser.add_argument(
            '--trustee',
            metavar='<trustee-user>',
            help=_('Trustee user to filter (name or ID)'),
        )
        parser.add_argument(
            '--trustor-domain',
            metavar='<trustor-domain>',
            help=_('Domain that contains <trustor> (name or ID)'),
        )
        parser.add_argument(
            '--trustee-domain',
            metavar='<trustee-domain>',
            help=_('Domain that contains <trustee> (name or ID)'),
        )
        parser.add_argument(
            '--auth-user',
            action="store_true",
            dest='authuser',
            help=_('Only list trusts related to the authenticated user'),
        )
        return parser

    def take_action(self, parsed_args):
        identity_client = self.app.client_manager.identity
        auth_ref = self.app.client_manager.auth_ref

        if parsed_args.authuser and any([
            parsed_args.trustor,
            parsed_args.trustor_domain,
            parsed_args.trustee,
            parsed_args.trustee_domain,
        ]):
            msg = _("--authuser cannot be used with --trustee or --trustor")
            raise exceptions.CommandError(msg)

        if parsed_args.trustee_domain and not parsed_args.trustee:
            msg = _("Using --trustee-domain mandates the use of --trustee")
            raise exceptions.CommandError(msg)

        if parsed_args.trustor_domain and not parsed_args.trustor:
            msg = _("Using --trustor-domain mandates the use of --trustor")
            raise exceptions.CommandError(msg)

        if parsed_args.authuser:
            if auth_ref:
                user = common.find_user(
                    identity_client,
                    auth_ref.user_id
                )
                # We need two calls here as we want trusts with
                # either the trustor or the trustee set to current user
                # using a single call would give us trusts with both
                # trustee and trustor set to current user
                data1 = identity_client.trusts.list(trustor_user=user)
                data2 = identity_client.trusts.list(trustee_user=user)
                data = set(data1 + data2)
        else:
            trustor = None
            if parsed_args.trustor:
                trustor = common.find_user(
                    identity_client,
                    parsed_args.trustor,
                    parsed_args.trustor_domain,
                )

            trustee = None
            if parsed_args.trustee:
                trustee = common.find_user(
                    identity_client,
                    parsed_args.trustor,
                    parsed_args.trustor_domain,
                )

            data = self.app.client_manager.identity.trusts.list(
                trustor_user=trustor,
                trustee_user=trustee,
            )

        columns = ('ID', 'Expires At', 'Impersonation', 'Project ID',
                   'Trustee User ID', 'Trustor User ID')

        return (columns,
                (utils.get_item_properties(
                    s, columns,
                    formatters={},
                ) for s in data))


class ShowTrust(command.ShowOne):
    _description = _("Display trust details")

    def get_parser(self, prog_name):
        parser = super(ShowTrust, self).get_parser(prog_name)
        parser.add_argument(
            'trust',
            metavar='<trust>',
            help=_('Trust to display'),
        )
        return parser

    def take_action(self, parsed_args):
        identity_client = self.app.client_manager.identity
        trust = utils.find_resource(identity_client.trusts,
                                    parsed_args.trust)

        trust._info.pop('roles_links', None)
        trust._info.pop('links', None)

        # Format roles into something sensible
        roles = trust._info.pop('roles')
        msg = ' '.join(r['name'] for r in roles)
        trust._info['roles'] = msg

        return zip(*sorted(trust._info.items()))



# Copyright (c) 2018 China Telecom Corporation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock
from unittest.mock import call

from osc_lib import exceptions

from openstackclient.network.v2 import floating_ip_port_forwarding
from openstackclient.tests.unit.identity.v2_0 import fakes as identity_fakes_v2
from openstackclient.tests.unit.network.v2 import fakes as network_fakes
from openstackclient.tests.unit import utils as tests_utils


class TestFloatingIPPortForwarding(network_fakes.TestNetworkV2):

    def setUp(self):
        super(TestFloatingIPPortForwarding, self).setUp()
        self.network = self.app.client_manager.network
        self.floating_ip = (network_fakes.FakeFloatingIP.
                            create_one_floating_ip())
        self.port = network_fakes.create_one_port()
        self.project = identity_fakes_v2.FakeProject.create_one_project()
        self.network.find_port = mock.Mock(return_value=self.port)


class TestCreateFloatingIPPortForwarding(TestFloatingIPPortForwarding):

    def setUp(self):
        super(TestCreateFloatingIPPortForwarding, self).setUp()
        self.new_port_forwarding = (
            network_fakes.FakeFloatingIPPortForwarding.
            create_one_port_forwarding(
                attrs={
                    'internal_port_id': self.port.id,
                    'floatingip_id': self.floating_ip.id,
                }
            )
        )

        self.new_port_forwarding_with_ranges = (
            network_fakes.FakeFloatingIPPortForwarding.
            create_one_port_forwarding(
                use_range=True,
                attrs={
                    'internal_port_id': self.port.id,
                    'floatingip_id': self.floating_ip.id,
                }
            )
        )

        self.network.create_floating_ip_port_forwarding = mock.Mock(
            return_value=self.new_port_forwarding)

        self.network.find_ip = mock.Mock(
            return_value=self.floating_ip
        )

        # Get the command object to test
        self.cmd = floating_ip_port_forwarding.CreateFloatingIPPortForwarding(
            self.app, self.namespace)

        self.columns = (
            'description',
            'external_port',
            'external_port_range',
            'floatingip_id',
            'id',
            'internal_ip_address',
            'internal_port',
            'internal_port_id',
            'internal_port_range',
            'protocol'
        )

        self.data = (
            self.new_port_forwarding.description,
            self.new_port_forwarding.external_port,
            self.new_port_forwarding.external_port_range,
            self.new_port_forwarding.floatingip_id,
            self.new_port_forwarding.id,
            self.new_port_forwarding.internal_ip_address,
            self.new_port_forwarding.internal_port,
            self.new_port_forwarding.internal_port_id,
            self.new_port_forwarding.internal_port_range,
            self.new_port_forwarding.protocol,
        )

    def test_create_no_options(self):
        arglist = []
        verifylist = []

        # Missing required args should bail here
        self.assertRaises(tests_utils.ParserException, self.check_parser,
                          self.cmd, arglist, verifylist)

    def test_create_all_options_with_range(self):
        arglist = [
            '--port', self.new_port_forwarding_with_ranges.internal_port_id,
            '--internal-protocol-port',
            self.new_port_forwarding_with_ranges.internal_port_range,
            '--external-protocol-port',
            self.new_port_forwarding_with_ranges.external_port_range,
            '--protocol', self.new_port_forwarding_with_ranges.protocol,
            self.new_port_forwarding_with_ranges.floatingip_id,
            '--internal-ip-address',
            self.new_port_forwarding_with_ranges.internal_ip_address,
            '--description',
            self.new_port_forwarding_with_ranges.description,
        ]
        verifylist = [
            ('port', self.new_port_forwarding_with_ranges.internal_port_id),
            ('internal_protocol_port',
             self.new_port_forwarding_with_ranges.internal_port_range),
            ('external_protocol_port',
             self.new_port_forwarding_with_ranges.external_port_range),
            ('protocol', self.new_port_forwarding_with_ranges.protocol),
            ('floating_ip',
             self.new_port_forwarding_with_ranges.floatingip_id),
            ('internal_ip_address', self.new_port_forwarding_with_ranges.
                internal_ip_address),
            ('description', self.new_port_forwarding_with_ranges.description),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        columns, data = self.cmd.take_action(parsed_args)

        self.network.create_floating_ip_port_forwarding.\
            assert_called_once_with(
                self.new_port_forwarding.floatingip_id,
                **{
                    'external_port_range':
                        self.new_port_forwarding_with_ranges.
                        external_port_range,
                    'internal_ip_address':
                        self.new_port_forwarding_with_ranges.
                    internal_ip_address,
                    'internal_port_range':
                        self.new_port_forwarding_with_ranges.
                        internal_port_range,
                    'internal_port_id':
                        self.new_port_forwarding_with_ranges.internal_port_id,
                    'protocol': self.new_port_forwarding_with_ranges.protocol,
                    'description':
                        self.new_port_forwarding_with_ranges.description,
                })
        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)

    def test_create_all_options_with_range_invalid_port_exception(self):
        invalid_port_range = '999999:999999'
        arglist = [
            '--port', self.new_port_forwarding_with_ranges.internal_port_id,
            '--internal-protocol-port', invalid_port_range,
            '--external-protocol-port', invalid_port_range,
            '--protocol', self.new_port_forwarding_with_ranges.protocol,
            self.new_port_forwarding_with_ranges.floatingip_id,
            '--internal-ip-address',
            self.new_port_forwarding_with_ranges.internal_ip_address,
            '--description',
            self.new_port_forwarding_with_ranges.description,
        ]
        verifylist = [
            ('port', self.new_port_forwarding_with_ranges.internal_port_id),
            ('internal_protocol_port', invalid_port_range),
            ('external_protocol_port', invalid_port_range),
            ('protocol', self.new_port_forwarding_with_ranges.protocol),
            ('floating_ip',
             self.new_port_forwarding_with_ranges.floatingip_id),
            ('internal_ip_address', self.new_port_forwarding_with_ranges.
                internal_ip_address),
            ('description', self.new_port_forwarding_with_ranges.description),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        msg = 'The port number range is <1-65535>'
        try:
            self.cmd.take_action(parsed_args)
            self.fail('CommandError should be raised.')
        except exceptions.CommandError as e:
            self.assertEqual(msg, str(e))
            self.network.create_floating_ip_port_forwarding.assert_not_called()

    def test_create_all_options_with_invalid_range_exception(self):
        invalid_port_range = '80:70'
        arglist = [
            '--port', self.new_port_forwarding_with_ranges.internal_port_id,
            '--internal-protocol-port', invalid_port_range,
            '--external-protocol-port', invalid_port_range,
            '--protocol', self.new_port_forwarding_with_ranges.protocol,
            self.new_port_forwarding_with_ranges.floatingip_id,
            '--internal-ip-address',
            self.new_port_forwarding_with_ranges.internal_ip_address,
            '--description',
            self.new_port_forwarding_with_ranges.description,
        ]
        verifylist = [
            ('port', self.new_port_forwarding_with_ranges.internal_port_id),
            ('internal_protocol_port', invalid_port_range),
            ('external_protocol_port', invalid_port_range),
            ('protocol', self.new_port_forwarding_with_ranges.protocol),
            ('floating_ip',
             self.new_port_forwarding_with_ranges.floatingip_id),
            ('internal_ip_address', self.new_port_forwarding_with_ranges.
                internal_ip_address),
            ('description', self.new_port_forwarding_with_ranges.description),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        msg = 'The last number in port range must be greater or equal to ' \
              'the first'
        try:
            self.cmd.take_action(parsed_args)
            self.fail('CommandError should be raised.')
        except exceptions.CommandError as e:
            self.assertEqual(msg, str(e))
            self.network.create_floating_ip_port_forwarding.assert_not_called()

    def test_create_all_options_with_unmatch_ranges_exception(self):
        internal_range = '80:90'
        external_range = '8080:8100'
        arglist = [
            '--port', self.new_port_forwarding_with_ranges.internal_port_id,
            '--internal-protocol-port', internal_range,
            '--external-protocol-port', external_range,
            '--protocol', self.new_port_forwarding_with_ranges.protocol,
            self.new_port_forwarding_with_ranges.floatingip_id,
            '--internal-ip-address',
            self.new_port_forwarding_with_ranges.internal_ip_address,
            '--description',
            self.new_port_forwarding_with_ranges.description,
        ]
        verifylist = [
            ('port', self.new_port_forwarding_with_ranges.internal_port_id),
            ('internal_protocol_port', internal_range),
            ('external_protocol_port', external_range),
            ('protocol', self.new_port_forwarding_with_ranges.protocol),
            ('floating_ip',
             self.new_port_forwarding_with_ranges.floatingip_id),
            ('internal_ip_address', self.new_port_forwarding_with_ranges.
                internal_ip_address),
            ('description', self.new_port_forwarding_with_ranges.description),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        msg = "The relation between internal and external ports does not " \
              "match the pattern 1:N and N:N"
        try:
            self.cmd.take_action(parsed_args)
            self.fail('CommandError should be raised.')
        except exceptions.CommandError as e:
            self.assertEqual(msg, str(e))
            self.network.create_floating_ip_port_forwarding.assert_not_called()

    def test_create_all_options(self):
        arglist = [
            '--port', self.new_port_forwarding.internal_port_id,
            '--internal-protocol-port',
            str(self.new_port_forwarding.internal_port),
            '--external-protocol-port',
            str(self.new_port_forwarding.external_port),
            '--protocol', self.new_port_forwarding.protocol,
            self.new_port_forwarding.floatingip_id,
            '--internal-ip-address',
            self.new_port_forwarding.internal_ip_address,
            '--description',
            self.new_port_forwarding.description,
        ]
        verifylist = [
            ('port', self.new_port_forwarding.internal_port_id),
            ('internal_protocol_port',
             str(self.new_port_forwarding.internal_port)),
            ('external_protocol_port',
             str(self.new_port_forwarding.external_port)),
            ('protocol', self.new_port_forwarding.protocol),
            ('floating_ip', self.new_port_forwarding.floatingip_id),
            ('internal_ip_address', self.new_port_forwarding.
                internal_ip_address),
            ('description', self.new_port_forwarding.description),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        columns, data = self.cmd.take_action(parsed_args)

        self.network.create_floating_ip_port_forwarding.\
            assert_called_once_with(
                self.new_port_forwarding.floatingip_id,
                **{
                    'external_port': self.new_port_forwarding.external_port,
                    'internal_ip_address': self.new_port_forwarding.
                    internal_ip_address,
                    'internal_port': self.new_port_forwarding.internal_port,
                    'internal_port_id': self.new_port_forwarding.
                    internal_port_id,
                    'protocol': self.new_port_forwarding.protocol,
                    'description': self.new_port_forwarding.description,
                })
        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)


class TestDeleteFloatingIPPortForwarding(TestFloatingIPPortForwarding):

    def setUp(self):
        super(TestDeleteFloatingIPPortForwarding, self).setUp()
        self._port_forwarding = (
            network_fakes.FakeFloatingIPPortForwarding.create_port_forwardings(
                count=2, attrs={
                    'floatingip_id': self.floating_ip.id,
                }
            )
        )
        self.network.delete_floating_ip_port_forwarding = mock.Mock(
            return_value=None
        )

        self.network.find_ip = mock.Mock(
            return_value=self.floating_ip
        )
        # Get the command object to test
        self.cmd = floating_ip_port_forwarding.DeleteFloatingIPPortForwarding(
            self.app, self.namespace)

    def test_port_forwarding_delete(self):
        arglist = [
            self.floating_ip.id,
            self._port_forwarding[0].id,
        ]
        verifylist = [
            ('floating_ip', self.floating_ip.id),
            ('port_forwarding_id', [self._port_forwarding[0].id]),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        result = self.cmd.take_action(parsed_args)

        self.network.delete_floating_ip_port_forwarding.\
            assert_called_once_with(
                self.floating_ip.id,
                self._port_forwarding[0].id,
                ignore_missing=False
            )

        self.assertIsNone(result)

    def test_multi_port_forwardings_delete(self):
        arglist = []
        pf_id = []

        arglist.append(str(self.floating_ip))

        for a in self._port_forwarding:
            arglist.append(a.id)
            pf_id.append(a.id)

        verifylist = [
            ('floating_ip', str(self.floating_ip)),
            ('port_forwarding_id', pf_id),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        result = self.cmd.take_action(parsed_args)

        calls = []
        for a in self._port_forwarding:
            calls.append(call(a.floatingip_id, a.id, ignore_missing=False))

        self.network.delete_floating_ip_port_forwarding.assert_has_calls(calls)
        self.assertIsNone(result)

    def test_multi_port_forwarding_delete_with_exception(self):
        arglist = [
            self.floating_ip.id,
            self._port_forwarding[0].id,
            'unexist_port_forwarding_id',
        ]
        verifylist = [
            ('floating_ip', self.floating_ip.id),
            ('port_forwarding_id',
             [self._port_forwarding[0].id, 'unexist_port_forwarding_id']),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        delete_mock_result = [None, exceptions.CommandError]

        self.network.delete_floating_ip_port_forwarding = (
            mock.MagicMock(side_effect=delete_mock_result)
        )

        try:
            self.cmd.take_action(parsed_args)
            self.fail('CommandError should be raised.')
        except exceptions.CommandError as e:
            self.assertEqual(
                '1 of 2 Port forwarding failed to delete.',
                str(e)
            )

        self.network.delete_floating_ip_port_forwarding.\
            assert_any_call(
                self.floating_ip.id,
                'unexist_port_forwarding_id',
                ignore_missing=False
            )
        self.network.delete_floating_ip_port_forwarding.\
            assert_any_call(
                self.floating_ip.id,
                self._port_forwarding[0].id,
                ignore_missing=False
            )


class TestListFloatingIPPortForwarding(TestFloatingIPPortForwarding):

    columns = (
        'ID',
        'Internal Port ID',
        'Internal IP Address',
        'Internal Port',
        'Internal Port Range',
        'External Port',
        'External Port Range',
        'Protocol',
        'Description',
    )

    def setUp(self):
        super(TestListFloatingIPPortForwarding, self).setUp()
        self.port_forwardings = (
            network_fakes.FakeFloatingIPPortForwarding.create_port_forwardings(
                count=3, attrs={
                    'internal_port_id': self.port.id,
                    'floatingip_id': self.floating_ip.id,
                }
            )
        )
        self.data = []
        for port_forwarding in self.port_forwardings:
            self.data.append((
                port_forwarding.id,
                port_forwarding.internal_port_id,
                port_forwarding.internal_ip_address,
                port_forwarding.internal_port,
                port_forwarding.internal_port_range,
                port_forwarding.external_port,
                port_forwarding.external_port_range,
                port_forwarding.protocol,
                port_forwarding.description,
            ))
        self.network.floating_ip_port_forwardings = mock.Mock(
            return_value=self.port_forwardings
        )
        self.network.find_ip = mock.Mock(
            return_value=self.floating_ip
        )
        # Get the command object to test
        self.cmd = floating_ip_port_forwarding.ListFloatingIPPortForwarding(
            self.app,
            self.namespace
        )

    def test_port_forwarding_list(self):
        arglist = [
            self.floating_ip.id
        ]
        verifylist = [
            ('floating_ip', self.floating_ip.id)
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        columns, data = self.cmd.take_action(parsed_args)

        self.network.floating_ip_port_forwardings.assert_called_once_with(
            self.floating_ip,
            **{}
        )
        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, list(data))

    def test_port_forwarding_list_all_options(self):
        arglist = [
            '--port', self.port_forwardings[0].internal_port_id,
            '--external-protocol-port',
            str(self.port_forwardings[0].external_port),
            '--protocol', self.port_forwardings[0].protocol,
            self.port_forwardings[0].floatingip_id,
        ]

        verifylist = [
            ('port', self.port_forwardings[0].internal_port_id),
            ('external_protocol_port',
             str(self.port_forwardings[0].external_port)),
            ('protocol', self.port_forwardings[0].protocol),
            ('floating_ip', self.port_forwardings[0].floatingip_id),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        columns, data = self.cmd.take_action(parsed_args)

        query = {
            'internal_port_id': self.port_forwardings[0].internal_port_id,
            'external_port': self.port_forwardings[0].external_port,
            'protocol': self.port_forwardings[0].protocol,
        }

        self.network.floating_ip_port_forwardings.assert_called_once_with(
            self.floating_ip,
            **query
        )
        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, list(data))


class TestSetFloatingIPPortForwarding(TestFloatingIPPortForwarding):

    # The Port Forwarding to set.
    def setUp(self):
        super(TestSetFloatingIPPortForwarding, self).setUp()
        self._port_forwarding = (
            network_fakes.FakeFloatingIPPortForwarding.
            create_one_port_forwarding(
                attrs={
                    'floatingip_id': self.floating_ip.id,
                }
            )
        )
        self.network.update_floating_ip_port_forwarding = mock.Mock(
            return_value=None
        )

        self.network.find_floating_ip_port_forwarding = mock.Mock(
            return_value=self._port_forwarding)
        self.network.find_ip = mock.Mock(
            return_value=self.floating_ip
        )
        # Get the command object to test
        self.cmd = floating_ip_port_forwarding.SetFloatingIPPortForwarding(
            self.app,
            self.namespace
        )

    def test_set_nothing(self):
        arglist = [
            self._port_forwarding.floatingip_id,
            self._port_forwarding.id,
        ]
        verifylist = [
            ('floating_ip', self._port_forwarding.floatingip_id),
            ('port_forwarding_id', self._port_forwarding.id),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        result = self.cmd.take_action(parsed_args)

        attrs = {}
        self.network.update_floating_ip_port_forwarding.assert_called_with(
            self._port_forwarding.floatingip_id,
            self._port_forwarding.id,
            **attrs
        )
        self.assertIsNone(result)

    def test_set_all_thing(self):
        arglist_single = [
            '--port', self.port.id,
            '--internal-ip-address', 'new_internal_ip_address',
            '--internal-protocol-port', '100',
            '--external-protocol-port', '200',
            '--protocol', 'tcp',
            '--description', 'some description',
            self._port_forwarding.floatingip_id,
            self._port_forwarding.id,
        ]
        arglist_range = list(arglist_single)
        arglist_range[5] = '100:110'
        arglist_range[7] = '200:210'
        verifylist_single = [
            ('port', self.port.id),
            ('internal_ip_address', 'new_internal_ip_address'),
            ('internal_protocol_port', '100'),
            ('external_protocol_port', '200'),
            ('protocol', 'tcp'),
            ('description', 'some description'),
            ('floating_ip', self._port_forwarding.floatingip_id),
            ('port_forwarding_id', self._port_forwarding.id),
        ]
        verifylist_range = list(verifylist_single)
        verifylist_range[2] = ('internal_protocol_port', '100:110')
        verifylist_range[3] = ('external_protocol_port', '200:210')
        attrs_single = {
            'internal_port_id': self.port.id,
            'internal_ip_address': 'new_internal_ip_address',
            'internal_port': 100,
            'external_port': 200,
            'protocol': 'tcp',
            'description': 'some description',
        }
        attrs_range = dict(attrs_single, internal_port_range='100:110',
                           external_port_range='200:210')
        attrs_range.pop('internal_port')
        attrs_range.pop('external_port')

        def run_and_validate(arglist, verifylist, attrs):
            parsed_args = self.check_parser(self.cmd, arglist, verifylist)

            result = self.cmd.take_action(parsed_args)

            self.network.update_floating_ip_port_forwarding.assert_called_with(
                self._port_forwarding.floatingip_id,
                self._port_forwarding.id,
                **attrs
            )
            self.assertIsNone(result)

        run_and_validate(arglist_single, verifylist_single, attrs_single)
        run_and_validate(arglist_range, verifylist_range, attrs_range)


class TestShowFloatingIPPortForwarding(TestFloatingIPPortForwarding):

    # The port forwarding to show.
    columns = (
        'description',
        'external_port',
        'external_port_range',
        'floatingip_id',
        'id',
        'internal_ip_address',
        'internal_port',
        'internal_port_id',
        'internal_port_range',
        'protocol',
    )

    def setUp(self):
        super(TestShowFloatingIPPortForwarding, self).setUp()
        self._port_forwarding = (
            network_fakes.FakeFloatingIPPortForwarding.
            create_one_port_forwarding(
                attrs={
                    'floatingip_id': self.floating_ip.id,
                }
            )
        )
        self.data = (
            self._port_forwarding.description,
            self._port_forwarding.external_port,
            self._port_forwarding.external_port_range,
            self._port_forwarding.floatingip_id,
            self._port_forwarding.id,
            self._port_forwarding.internal_ip_address,
            self._port_forwarding.internal_port,
            self._port_forwarding.internal_port_id,
            self._port_forwarding.internal_port_range,
            self._port_forwarding.protocol,
        )
        self.network.find_floating_ip_port_forwarding = mock.Mock(
            return_value=self._port_forwarding
        )
        self.network.find_ip = mock.Mock(
            return_value=self.floating_ip
        )
        # Get the command object to test
        self.cmd = floating_ip_port_forwarding.ShowFloatingIPPortForwarding(
            self.app,
            self.namespace
        )

    def test_show_no_options(self):
        arglist = []
        verifylist = []

        # Missing required args should bail here
        self.assertRaises(tests_utils.ParserException, self.check_parser,
                          self.cmd, arglist, verifylist)

    def test_show_default_options(self):
        arglist = [
            self._port_forwarding.floatingip_id,
            self._port_forwarding.id,
        ]
        verifylist = [
            ('floating_ip', self._port_forwarding.floatingip_id),
            ('port_forwarding_id', self._port_forwarding.id),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        columns, data = self.cmd.take_action(parsed_args)

        self.network.find_floating_ip_port_forwarding.assert_called_once_with(
            self.floating_ip,
            self._port_forwarding.id,
            ignore_missing=False
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(list(self.data), list(data))



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#

import copy
from unittest import mock

from osc_lib import exceptions
from osc_lib import utils

from openstackclient.identity.v3 import trust
from openstackclient.tests.unit import fakes
from openstackclient.tests.unit.identity.v3 import fakes as identity_fakes


class TestTrust(identity_fakes.TestIdentityv3):

    def setUp(self):
        super(TestTrust, self).setUp()

        self.trusts_mock = self.app.client_manager.identity.trusts
        self.trusts_mock.reset_mock()
        self.projects_mock = self.app.client_manager.identity.projects
        self.projects_mock.reset_mock()
        self.users_mock = self.app.client_manager.identity.users
        self.users_mock.reset_mock()
        self.roles_mock = self.app.client_manager.identity.roles
        self.roles_mock.reset_mock()


class TestTrustCreate(TestTrust):

    def setUp(self):
        super(TestTrustCreate, self).setUp()

        self.projects_mock.get.return_value = fakes.FakeResource(
            None,
            copy.deepcopy(identity_fakes.PROJECT),
            loaded=True,
        )

        self.users_mock.get.return_value = fakes.FakeResource(
            None,
            copy.deepcopy(identity_fakes.USER),
            loaded=True,
        )

        self.roles_mock.get.return_value = fakes.FakeResource(
            None,
            copy.deepcopy(identity_fakes.ROLE),
            loaded=True,
        )

        self.trusts_mock.create.return_value = fakes.FakeResource(
            None,
            copy.deepcopy(identity_fakes.TRUST),
            loaded=True,
        )

        # Get the command object to test
        self.cmd = trust.CreateTrust(self.app, None)

    def test_trust_create_basic(self):
        arglist = [
            '--project', identity_fakes.project_id,
            '--role', identity_fakes.role_id,
            identity_fakes.user_id,
            identity_fakes.user_id
        ]
        verifylist = [
            ('project', identity_fakes.project_id),
            ('impersonate', False),
            ('role', [identity_fakes.role_id]),
            ('trustor', identity_fakes.user_id),
            ('trustee', identity_fakes.user_id),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # In base command class ShowOne in cliff, abstract method take_action()
        # returns a two-part tuple with a tuple of column names and a tuple of
        # data to be shown.
        columns, data = self.cmd.take_action(parsed_args)

        # Set expected values
        kwargs = {
            'impersonation': False,
            'project': identity_fakes.project_id,
            'role_ids': [identity_fakes.role_id],
            'expires_at': None,
        }
        # TrustManager.create(trustee_id, trustor_id, impersonation=,
        #   project=, role_names=, expires_at=)
        self.trusts_mock.create.assert_called_with(
            identity_fakes.user_id,
            identity_fakes.user_id,
            **kwargs
        )

        collist = ('expires_at', 'id', 'impersonation', 'project_id',
                   'roles', 'trustee_user_id', 'trustor_user_id')
        self.assertEqual(collist, columns)
        datalist = (
            identity_fakes.trust_expires,
            identity_fakes.trust_id,
            identity_fakes.trust_impersonation,
            identity_fakes.project_id,
            identity_fakes.role_name,
            identity_fakes.user_id,
            identity_fakes.user_id
        )
        self.assertEqual(datalist, data)


class TestTrustDelete(TestTrust):

    def setUp(self):
        super(TestTrustDelete, self).setUp()

        # This is the return value for utils.find_resource()
        self.trusts_mock.get.return_value = fakes.FakeResource(
            None,
            copy.deepcopy(identity_fakes.TRUST),
            loaded=True,
        )
        self.trusts_mock.delete.return_value = None

        # Get the command object to test
        self.cmd = trust.DeleteTrust(self.app, None)

    def test_trust_delete(self):
        arglist = [
            identity_fakes.trust_id,
        ]
        verifylist = [
            ('trust', [identity_fakes.trust_id])
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        result = self.cmd.take_action(parsed_args)

        self.trusts_mock.delete.assert_called_with(
            identity_fakes.trust_id,
        )
        self.assertIsNone(result)

    @mock.patch.object(utils, 'find_resource')
    def test_delete_multi_trusts_with_exception(self, find_mock):
        find_mock.side_effect = [self.trusts_mock.get.return_value,
                                 exceptions.CommandError]
        arglist = [
            identity_fakes.trust_id,
            'unexist_trust',
        ]
        verifylist = [
            ('trust', arglist),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        try:
            self.cmd.take_action(parsed_args)
            self.fail('CommandError should be raised.')
        except exceptions.CommandError as e:
            self.assertEqual('1 of 2 trusts failed to delete.',
                             str(e))

        find_mock.assert_any_call(self.trusts_mock, identity_fakes.trust_id)
        find_mock.assert_any_call(self.trusts_mock, 'unexist_trust')

        self.assertEqual(2, find_mock.call_count)
        self.trusts_mock.delete.assert_called_once_with(
            identity_fakes.trust_id)


class TestTrustList(TestTrust):

    def setUp(self):
        super(TestTrustList, self).setUp()

        self.trusts_mock.list.return_value = [
            fakes.FakeResource(
                None,
                copy.deepcopy(identity_fakes.TRUST),
                loaded=True,
            ),
        ]

        # Get the command object to test
        self.cmd = trust.ListTrust(self.app, None)

    def test_trust_list_no_options(self):
        arglist = []
        verifylist = []
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # In base command class Lister in cliff, abstract method take_action()
        # returns a tuple containing the column names and an iterable
        # containing the data to be listed.
        columns, data = self.cmd.take_action(parsed_args)

        self.trusts_mock.list.assert_called_with(
            trustor_user=None,
            trustee_user=None,
        )

        collist = ('ID', 'Expires At', 'Impersonation', 'Project ID',
                   'Trustee User ID', 'Trustor User ID')
        self.assertEqual(collist, columns)
        datalist = ((
            identity_fakes.trust_id,
            identity_fakes.trust_expires,
            identity_fakes.trust_impersonation,
            identity_fakes.project_id,
            identity_fakes.user_id,
            identity_fakes.user_id
        ), )
        self.assertEqual(datalist, tuple(data))

    def test_trust_list_auth_user(self):
        auth_ref = self.app.client_manager.auth_ref = mock.Mock()
        auth_ref.user_id.return_value = identity_fakes.user_id

        arglist = ['--auth-user']
        verifylist = [
            ('trustor', None),
            ('trustee', None),
            ('authuser', True),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # In base command class Lister in cliff, abstract method take_action()
        # returns a tuple containing the column names and an iterable
        # containing the data to be listed.
        columns, data = self.cmd.take_action(parsed_args)

        self.trusts_mock.list.assert_any_call(
            trustor_user=self.users_mock.get()
        )
        self.trusts_mock.list.assert_any_call(
            trustee_user=self.users_mock.get()
        )

        collist = ('ID', 'Expires At', 'Impersonation', 'Project ID',
                   'Trustee User ID', 'Trustor User ID')
        self.assertEqual(collist, columns)
        datalist = ((
            identity_fakes.trust_id,
            identity_fakes.trust_expires,
            identity_fakes.trust_impersonation,
            identity_fakes.project_id,
            identity_fakes.user_id,
            identity_fakes.user_id
        ), )
        self.assertEqual(datalist, tuple(data))

    def test_trust_list_trustee(self):
        arglist = ['--trustee', identity_fakes.user_name]
        verifylist = [
            ('trustor', None),
            ('trustee', identity_fakes.user_name),
            ('authuser', False),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # In base command class Lister in cliff, abstract method take_action()
        # returns a tuple containing the column names and an iterable
        # containing the data to be listed.
        columns, data = self.cmd.take_action(parsed_args)

        print(self.trusts_mock.list.call_args_list)
        self.trusts_mock.list.assert_any_call(
            trustee_user=self.users_mock.get(),
            trustor_user=None,
        )

        collist = ('ID', 'Expires At', 'Impersonation', 'Project ID',
                   'Trustee User ID', 'Trustor User ID')
        self.assertEqual(collist, columns)
        datalist = ((
            identity_fakes.trust_id,
            identity_fakes.trust_expires,
            identity_fakes.trust_impersonation,
            identity_fakes.project_id,
            identity_fakes.user_id,
            identity_fakes.user_id
        ), )
        self.assertEqual(datalist, tuple(data))

    def test_trust_list_trustor(self):
        arglist = ['--trustor', identity_fakes.user_name]
        verifylist = [
            ('trustee', None),
            ('trustor', identity_fakes.user_name),
            ('authuser', False),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # In base command class Lister in cliff, abstract method take_action()
        # returns a tuple containing the column names and an iterable
        # containing the data to be listed.
        columns, data = self.cmd.take_action(parsed_args)

        print(self.trusts_mock.list.call_args_list)
        self.trusts_mock.list.assert_any_call(
            trustor_user=self.users_mock.get(),
            trustee_user=None,
        )

        collist = ('ID', 'Expires At', 'Impersonation', 'Project ID',
                   'Trustee User ID', 'Trustor User ID')
        self.assertEqual(collist, columns)
        datalist = ((
            identity_fakes.trust_id,
            identity_fakes.trust_expires,
            identity_fakes.trust_impersonation,
            identity_fakes.project_id,
            identity_fakes.user_id,
            identity_fakes.user_id
        ), )
        self.assertEqual(datalist, tuple(data))


class TestTrustShow(TestTrust):

    def setUp(self):
        super(TestTrustShow, self).setUp()

        self.trusts_mock.get.return_value = fakes.FakeResource(
            None,
            copy.deepcopy(identity_fakes.TRUST),
            loaded=True,
        )

        # Get the command object to test
        self.cmd = trust.ShowTrust(self.app, None)

    def test_trust_show(self):
        arglist = [
            identity_fakes.trust_id,
        ]
        verifylist = [
            ('trust', identity_fakes.trust_id),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # In base command class ShowOne in cliff, abstract method take_action()
        # returns a two-part tuple with a tuple of column names and a tuple of
        # data to be shown.
        columns, data = self.cmd.take_action(parsed_args)

        self.trusts_mock.get.assert_called_with(identity_fakes.trust_id)

        collist = ('expires_at', 'id', 'impersonation', 'project_id',
                   'roles', 'trustee_user_id', 'trustor_user_id')
        self.assertEqual(collist, columns)
        datalist = (
            identity_fakes.trust_expires,
            identity_fakes.trust_id,
            identity_fakes.trust_impersonation,
            identity_fakes.project_id,
            identity_fakes.role_name,
            identity_fakes.user_id,
            identity_fakes.user_id
        )
        self.assertEqual(datalist, data)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#

"""Flavor action implementations"""

import logging

from osc_lib.command import command
from osc_lib import exceptions
from osc_lib import utils

from openstackclient.i18n import _
from openstackclient.identity import common as identity_common
from openstackclient.network import common

LOG = logging.getLogger(__name__)


def _get_columns(item):
    column_map = {
        'is_enabled': 'enabled',
    }

    hidden_columns = ['location', 'tenant_id']
    return utils.get_osc_show_columns_for_sdk_resource(
        item,
        column_map,
        hidden_columns
    )


def _get_attrs(client_manager, parsed_args):
    attrs = {}
    attrs['name'] = parsed_args.name
    attrs['service_type'] = parsed_args.service_type
    if parsed_args.description is not None:
        attrs['description'] = parsed_args.description
    if parsed_args.enable:
        attrs['enabled'] = True
    if parsed_args.disable:
        attrs['enabled'] = False
    if 'project' in parsed_args and parsed_args.project is not None:
        identity_client = client_manager.identity
        project_id = identity_common.find_project(
            identity_client,
            parsed_args.project,
            parsed_args.project_domain,
        ).id
        attrs['project_id'] = project_id

    return attrs


class AddNetworkFlavorToProfile(command.Command):
    _description = _("Add a service profile to a network flavor")

    def get_parser(self, prog_name):
        parser = super(
            AddNetworkFlavorToProfile, self).get_parser(prog_name)
        parser.add_argument(
            'flavor',
            metavar="<flavor>",
            help=_("Network flavor (name or ID)")
        )
        parser.add_argument(
            'service_profile',
            metavar="<service-profile>",
            help=_("Service profile (ID only)")
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.network
        obj_flavor = client.find_flavor(
            parsed_args.flavor, ignore_missing=False)
        obj_service_profile = client.find_service_profile(
            parsed_args.service_profile, ignore_missing=False)
        client.associate_flavor_with_service_profile(
            obj_flavor, obj_service_profile)


# TODO(dasanind): Use the SDK resource mapped attribute names once the
# OSC minimum requirements include SDK 1.0.
class CreateNetworkFlavor(command.ShowOne, common.NeutronCommandWithExtraArgs):
    _description = _("Create new network flavor")

    def get_parser(self, prog_name):
        parser = super(CreateNetworkFlavor, self).get_parser(prog_name)
        parser.add_argument(
            'name',
            metavar="<name>",
            help=_("Name for the flavor")
        )
        parser.add_argument(
            '--service-type',
            metavar="<service-type>",
            required=True,
            help=_('Service type to which the flavor applies to: e.g. VPN '
                   '(See openstack network service provider list for loaded '
                   'examples.)')
        )
        parser.add_argument(
            '--description',
            help=_('Description for the flavor')
        )
        parser.add_argument(
            '--project',
            metavar="<project>",
            help=_("Owner's project (name or ID)")
        )
        identity_common.add_project_domain_option_to_parser(parser)

        enable_group = parser.add_mutually_exclusive_group()
        enable_group.add_argument(
            '--enable',
            action='store_true',
            help=_("Enable the flavor (default)")
        )
        enable_group.add_argument(
            '--disable',
            action='store_true',
            help=_("Disable the flavor")
        )

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.network
        attrs = _get_attrs(self.app.client_manager, parsed_args)
        attrs.update(
            self._parse_extra_properties(parsed_args.extra_properties))
        obj = client.create_flavor(**attrs)
        display_columns, columns = _get_columns(obj)
        data = utils.get_item_properties(obj, columns, formatters={})

        return (display_columns, data)


class DeleteNetworkFlavor(command.Command):
    _description = _("Delete network flavors")

    def get_parser(self, prog_name):
        parser = super(DeleteNetworkFlavor, self).get_parser(prog_name)

        parser.add_argument(
            'flavor',
            metavar='<flavor>',
            nargs='+',
            help=_('Flavor(s) to delete (name or ID)')
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.network
        result = 0

        for flavor in parsed_args.flavor:
            try:
                obj = client.find_flavor(flavor, ignore_missing=False)
                client.delete_flavor(obj)
            except Exception as e:
                result += 1
                LOG.error(_("Failed to delete flavor with "
                            "name or ID '%(flavor)s': %(e)s"),
                          {"flavor": flavor, "e": e})
        if result > 0:
            total = len(parsed_args.flavor)
            msg = (_("%(result)s of %(total)s flavors failed "
                     "to delete.") % {"result": result, "total": total})
            raise exceptions.CommandError(msg)


class ListNetworkFlavor(command.Lister):
    _description = _("List network flavors")

    def take_action(self, parsed_args):
        client = self.app.client_manager.network

        columns = (
            'id',
            'name',
            'is_enabled',
            'service_type',
            'description'
        )
        column_headers = (
            'ID',
            'Name',
            'Enabled',
            'Service Type',
            'Description'
        )

        data = client.flavors()
        return (column_headers,
                (utils.get_item_properties(
                    s, columns,
                ) for s in data))


class RemoveNetworkFlavorFromProfile(command.Command):
    _description = _(
        "Remove service profile from network flavor")

    def get_parser(self, prog_name):
        parser = super(
            RemoveNetworkFlavorFromProfile, self).get_parser(prog_name)
        parser.add_argument(
            'flavor',
            metavar="<flavor>",
            help=_("Network flavor (name or ID)")
        )
        parser.add_argument(
            'service_profile',
            metavar="<service-profile>",
            help=_("Service profile (ID only)")
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.network
        obj_flavor = client.find_flavor(
            parsed_args.flavor, ignore_missing=False)
        obj_service_profile = client.find_service_profile(
            parsed_args.service_profile, ignore_missing=False)
        client.disassociate_flavor_from_service_profile(
            obj_flavor, obj_service_profile)


# TODO(dasanind): Use only the SDK resource mapped attribute names once the
# OSC minimum requirements include SDK 1.0.
class SetNetworkFlavor(common.NeutronCommandWithExtraArgs):
    _description = _("Set network flavor properties")

    def get_parser(self, prog_name):
        parser = super(SetNetworkFlavor, self).get_parser(prog_name)
        parser.add_argument(
            'flavor',
            metavar="<flavor>",
            help=_("Flavor to update (name or ID)")
        )
        parser.add_argument(
            '--description',
            help=_('Set network flavor description')
        )
        enable_group = parser.add_mutually_exclusive_group()
        enable_group.add_argument(
            '--disable',
            action='store_true',
            help=_("Disable network flavor")
        )
        enable_group.add_argument(
            '--enable',
            action='store_true',
            help=_("Enable network flavor")
        )
        parser.add_argument(
            '--name',
            metavar="<name>",
            help=_('Set flavor name')
        )

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.network
        obj = client.find_flavor(
            parsed_args.flavor,
            ignore_missing=False)
        attrs = {}
        if parsed_args.name is not None:
            attrs['name'] = parsed_args.name
        if parsed_args.description is not None:
            attrs['description'] = parsed_args.description
        if parsed_args.enable:
            attrs['enabled'] = True
        if parsed_args.disable:
            attrs['enabled'] = False
        attrs.update(
            self._parse_extra_properties(parsed_args.extra_properties))
        client.update_flavor(obj, **attrs)


class ShowNetworkFlavor(command.ShowOne):
    _description = _("Display network flavor details")

    def get_parser(self, prog_name):
        parser = super(ShowNetworkFlavor, self).get_parser(prog_name)
        parser.add_argument(
            'flavor',
            metavar='<flavor>',
            help=_('Flavor to display (name or ID)')
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.network
        obj = client.find_flavor(parsed_args.flavor, ignore_missing=False)
        display_columns, columns = _get_columns(obj)
        data = utils.get_item_properties(obj, columns)
        return display_columns, data



#   Copyright 2014 CERN.
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.

import copy

from openstackclient.identity.v3 import service_provider
from openstackclient.tests.unit import fakes
from openstackclient.tests.unit.identity.v3 import fakes as service_fakes


class TestServiceProvider(service_fakes.TestFederatedIdentity):

    def setUp(self):
        super(TestServiceProvider, self).setUp()

        federation_lib = self.app.client_manager.identity.federation
        self.service_providers_mock = federation_lib.service_providers
        self.service_providers_mock.reset_mock()


class TestServiceProviderCreate(TestServiceProvider):

    columns = (
        'auth_url',
        'description',
        'enabled',
        'id',
        'sp_url',
    )
    datalist = (
        service_fakes.sp_auth_url,
        service_fakes.sp_description,
        True,
        service_fakes.sp_id,
        service_fakes.service_provider_url
    )

    def setUp(self):
        super(TestServiceProviderCreate, self).setUp()

        copied_sp = copy.deepcopy(service_fakes.SERVICE_PROVIDER)
        resource = fakes.FakeResource(None, copied_sp, loaded=True)
        self.service_providers_mock.create.return_value = resource
        self.cmd = service_provider.CreateServiceProvider(self.app, None)

    def test_create_service_provider_required_options_only(self):
        arglist = [
            '--auth-url', service_fakes.sp_auth_url,
            '--service-provider-url', service_fakes.service_provider_url,
            service_fakes.sp_id,
        ]
        verifylist = [
            ('auth_url', service_fakes.sp_auth_url),
            ('service_provider_url', service_fakes.service_provider_url),
            ('service_provider_id', service_fakes.sp_id),

        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        columns, data = self.cmd.take_action(parsed_args)

        # Set expected values
        kwargs = {
            'enabled': True,
            'description': None,
            'auth_url': service_fakes.sp_auth_url,
            'sp_url': service_fakes.service_provider_url
        }

        self.service_providers_mock.create.assert_called_with(
            id=service_fakes.sp_id,
            **kwargs
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.datalist, data)

    def test_create_service_provider_description(self):

        arglist = [
            '--description', service_fakes.sp_description,
            '--auth-url', service_fakes.sp_auth_url,
            '--service-provider-url', service_fakes.service_provider_url,
            service_fakes.sp_id,
        ]
        verifylist = [
            ('description', service_fakes.sp_description),
            ('auth_url', service_fakes.sp_auth_url),
            ('service_provider_url', service_fakes.service_provider_url),
            ('service_provider_id', service_fakes.sp_id),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        columns, data = self.cmd.take_action(parsed_args)

        # Set expected values
        kwargs = {
            'description': service_fakes.sp_description,
            'auth_url': service_fakes.sp_auth_url,
            'sp_url': service_fakes.service_provider_url,
            'enabled': True,
        }

        self.service_providers_mock.create.assert_called_with(
            id=service_fakes.sp_id,
            **kwargs
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.datalist, data)

    def test_create_service_provider_disabled(self):

        # Prepare FakeResource object
        service_provider = copy.deepcopy(service_fakes.SERVICE_PROVIDER)
        service_provider['enabled'] = False
        service_provider['description'] = None

        resource = fakes.FakeResource(None, service_provider, loaded=True)
        self.service_providers_mock.create.return_value = resource

        arglist = [
            '--auth-url', service_fakes.sp_auth_url,
            '--service-provider-url', service_fakes.service_provider_url,
            '--disable',
            service_fakes.sp_id,
        ]
        verifylist = [
            ('auth_url', service_fakes.sp_auth_url),
            ('service_provider_url', service_fakes.service_provider_url),
            ('service_provider_id', service_fakes.sp_id),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        columns, data = self.cmd.take_action(parsed_args)
        # Set expected values
        kwargs = {
            'auth_url': service_fakes.sp_auth_url,
            'sp_url': service_fakes.service_provider_url,
            'enabled': False,
            'description': None,
        }

        self.service_providers_mock.create.assert_called_with(
            id=service_fakes.sp_id,
            **kwargs
        )
        self.assertEqual(self.columns, columns)
        datalist = (
            service_fakes.sp_auth_url,
            None,
            False,
            service_fakes.sp_id,
            service_fakes.service_provider_url
        )
        self.assertEqual(datalist, data)


class TestServiceProviderDelete(TestServiceProvider):

    def setUp(self):
        super(TestServiceProviderDelete, self).setUp()

        # This is the return value for utils.find_resource()
        self.service_providers_mock.get.return_value = fakes.FakeResource(
            None,
            copy.deepcopy(service_fakes.SERVICE_PROVIDER),
            loaded=True,
        )

        self.service_providers_mock.delete.return_value = None
        self.cmd = service_provider.DeleteServiceProvider(self.app, None)

    def test_delete_service_provider(self):
        arglist = [
            service_fakes.sp_id,
        ]
        verifylist = [
            ('service_provider', [service_fakes.sp_id]),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        result = self.cmd.take_action(parsed_args)

        self.service_providers_mock.delete.assert_called_with(
            service_fakes.sp_id,
        )
        self.assertIsNone(result)


class TestServiceProviderList(TestServiceProvider):

    def setUp(self):
        super(TestServiceProviderList, self).setUp()

        self.service_providers_mock.get.return_value = fakes.FakeResource(
            None,
            copy.deepcopy(service_fakes.SERVICE_PROVIDER),
            loaded=True,
        )
        self.service_providers_mock.list.return_value = [
            fakes.FakeResource(
                None,
                copy.deepcopy(service_fakes.SERVICE_PROVIDER),
                loaded=True,
            ),
        ]

        # Get the command object to test
        self.cmd = service_provider.ListServiceProvider(self.app, None)

    def test_service_provider_list_no_options(self):
        arglist = []
        verifylist = []
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # In base command class Lister in cliff, abstract method take_action()
        # returns a tuple containing the column names and an iterable
        # containing the data to be listed.
        columns, data = self.cmd.take_action(parsed_args)

        self.service_providers_mock.list.assert_called_with()

        collist = ('ID', 'Enabled', 'Description', 'Auth URL')
        self.assertEqual(collist, columns)
        datalist = ((
            service_fakes.sp_id,
            True,
            service_fakes.sp_description,
            service_fakes.sp_auth_url
        ), )
        self.assertEqual(tuple(data), datalist)


class TestServiceProviderSet(TestServiceProvider):

    columns = (
        'auth_url',
        'description',
        'enabled',
        'id',
        'sp_url',
    )
    datalist = (
        service_fakes.sp_auth_url,
        service_fakes.sp_description,
        False,
        service_fakes.sp_id,
        service_fakes.service_provider_url,
    )

    def setUp(self):
        super(TestServiceProviderSet, self).setUp()
        self.cmd = service_provider.SetServiceProvider(self.app, None)

    def test_service_provider_disable(self):
        """Disable Service Provider

        Set Service Provider's ``enabled`` attribute to False.
        """

        def prepare(self):
            """Prepare fake return objects before the test is executed"""
            updated_sp = copy.deepcopy(service_fakes.SERVICE_PROVIDER)
            updated_sp['enabled'] = False
            resources = fakes.FakeResource(
                None,
                updated_sp,
                loaded=True
            )
            self.service_providers_mock.update.return_value = resources

        prepare(self)
        arglist = [
            '--disable', service_fakes.sp_id,
        ]
        verifylist = [
            ('service_provider', service_fakes.sp_id),
            ('enable', False),
            ('disable', True),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)
        self.service_providers_mock.update.assert_called_with(
            service_fakes.sp_id,
            enabled=False,
            description=None,
            auth_url=None,
            sp_url=None
        )

    def test_service_provider_enable(self):
        """Enable Service Provider.

        Set Service Provider's ``enabled`` attribute to True.
        """

        def prepare(self):
            """Prepare fake return objects before the test is executed"""
            resources = fakes.FakeResource(
                None,
                copy.deepcopy(service_fakes.SERVICE_PROVIDER),
                loaded=True
            )
            self.service_providers_mock.update.return_value = resources

        prepare(self)
        arglist = [
            '--enable', service_fakes.sp_id,
        ]
        verifylist = [
            ('service_provider', service_fakes.sp_id),
            ('enable', True),
            ('disable', False),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.cmd.take_action(parsed_args)
        self.service_providers_mock.update.assert_called_with(
            service_fakes.sp_id, enabled=True, description=None,
            auth_url=None, sp_url=None)

    def test_service_provider_no_options(self):
        def prepare(self):
            """Prepare fake return objects before the test is executed"""
            resources = fakes.FakeResource(
                None,
                copy.deepcopy(service_fakes.SERVICE_PROVIDER),
                loaded=True
            )
            self.service_providers_mock.get.return_value = resources

            resources = fakes.FakeResource(
                None,
                copy.deepcopy(service_fakes.SERVICE_PROVIDER),
                loaded=True,
            )
            self.service_providers_mock.update.return_value = resources

        prepare(self)
        arglist = [
            service_fakes.sp_id,
        ]
        verifylist = [
            ('service_provider', service_fakes.sp_id),
            ('description', None),
            ('enable', False),
            ('disable', False),
            ('auth_url', None),
            ('service_provider_url', None),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.cmd.take_action(parsed_args)


class TestServiceProviderShow(TestServiceProvider):

    def setUp(self):
        super(TestServiceProviderShow, self).setUp()

        ret = fakes.FakeResource(
            None,
            copy.deepcopy(service_fakes.SERVICE_PROVIDER),
            loaded=True,
        )
        self.service_providers_mock.get.side_effect = [Exception("Not found"),
                                                       ret]
        self.service_providers_mock.get.return_value = ret

        # Get the command object to test
        self.cmd = service_provider.ShowServiceProvider(self.app, None)

    def test_service_provider_show(self):
        arglist = [
            service_fakes.sp_id,
        ]
        verifylist = [
            ('service_provider', service_fakes.sp_id),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        self.service_providers_mock.get.assert_called_with(
            service_fakes.sp_id,
            id='BETA'
        )

        collist = ('auth_url', 'description', 'enabled', 'id', 'sp_url')
        self.assertEqual(collist, columns)
        datalist = (
            service_fakes.sp_auth_url,
            service_fakes.sp_description,
            True,
            service_fakes.sp_id,
            service_fakes.service_provider_url
        )
        self.assertEqual(data, datalist)



#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#

"""Volume V3 Volume action implementations"""

import logging

from cinderclient import api_versions
from osc_lib.cli import format_columns
from osc_lib.command import command
from osc_lib import exceptions
from osc_lib import utils

from openstackclient.i18n import _


LOG = logging.getLogger(__name__)


class VolumeSummary(command.ShowOne):
    _description = _("Show a summary of all volumes in this deployment.")

    def get_parser(self, prog_name):
        parser = super().get_parser(prog_name)
        parser.add_argument(
            '--all-projects',
            action='store_true',
            default=False,
            help=_('Include all projects (admin only)'),
        )
        return parser

    def take_action(self, parsed_args):

        volume_client = self.app.client_manager.volume

        if volume_client.api_version < api_versions.APIVersion('3.12'):
            msg = _(
                "--os-volume-api-version 3.12 or greater is required to "
                "support the 'volume summary' command"
            )
            raise exceptions.CommandError(msg)

        columns = [
            'total_count',
            'total_size',
        ]
        column_headers = [
            'Total Count',
            'Total Size',
        ]
        if volume_client.api_version.matches('3.36'):
            columns.append('metadata')
            column_headers.append('Metadata')

        # set value of 'all_tenants' when using project option
        all_projects = parsed_args.all_projects

        vol_summary = volume_client.volumes.summary(
            all_tenants=all_projects,
        )

        return (
            column_headers,
            utils.get_dict_properties(
                vol_summary['volume-summary'],
                columns,
                formatters={'metadata': format_columns.DictColumn},
            ),
        )


class VolumeRevertToSnapshot(command.Command):
    _description = _("Revert a volume to a snapshot.")

    def get_parser(self, prog_name):
        parser = super().get_parser(prog_name)
        parser.add_argument(
            'snapshot',
            metavar="<snapshot>",
            help=_('Name or ID of the snapshot to restore. The snapshot must '
                   'be the most recent one known to cinder.'),
        )
        return parser

    def take_action(self, parsed_args):

        volume_client = self.app.client_manager.volume

        if volume_client.api_version < api_versions.APIVersion('3.40'):
            msg = _(
                "--os-volume-api-version 3.40 or greater is required to "
                "support the 'volume revert snapshot' command"
            )
            raise exceptions.CommandError(msg)

        snapshot = utils.find_resource(
            volume_client.volume_snapshots, parsed_args.snapshot)
        volume = utils.find_resource(
            volume_client.volumes, snapshot.volume_id)

        volume_client.volumes.revert_to_snapshot(
            volume=volume, snapshot=snapshot)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.

import copy

from openstackclient.identity.v3 import consumer
from openstackclient.tests.unit import fakes
from openstackclient.tests.unit.identity.v3 import fakes as identity_fakes


class TestOAuth1(identity_fakes.TestOAuth1):

    def setUp(self):
        super(TestOAuth1, self).setUp()
        identity_client = self.app.client_manager.identity
        self.consumers_mock = identity_client.oauth1.consumers
        self.consumers_mock.reset_mock()


class TestConsumerCreate(TestOAuth1):

    def setUp(self):
        super(TestConsumerCreate, self).setUp()

        self.consumers_mock.create.return_value = fakes.FakeResource(
            None,
            copy.deepcopy(identity_fakes.OAUTH_CONSUMER),
            loaded=True,
        )

        self.cmd = consumer.CreateConsumer(self.app, None)

    def test_create_consumer(self):
        arglist = [
            '--description', identity_fakes.consumer_description,
        ]
        verifylist = [
            ('description', identity_fakes.consumer_description),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        columns, data = self.cmd.take_action(parsed_args)

        self.consumers_mock.create.assert_called_with(
            identity_fakes.consumer_description,
        )

        collist = ('description', 'id', 'secret')
        self.assertEqual(collist, columns)
        datalist = (
            identity_fakes.consumer_description,
            identity_fakes.consumer_id,
            identity_fakes.consumer_secret,
        )
        self.assertEqual(datalist, data)


class TestConsumerDelete(TestOAuth1):

    def setUp(self):
        super(TestConsumerDelete, self).setUp()

        # This is the return value for utils.find_resource()
        self.consumers_mock.get.return_value = fakes.FakeResource(
            None,
            copy.deepcopy(identity_fakes.OAUTH_CONSUMER),
            loaded=True,
        )

        self.consumers_mock.delete.return_value = None
        self.cmd = consumer.DeleteConsumer(self.app, None)

    def test_delete_consumer(self):
        arglist = [
            identity_fakes.consumer_id,
        ]
        verifylist = [
            ('consumer', [identity_fakes.consumer_id]),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        result = self.cmd.take_action(parsed_args)

        self.consumers_mock.delete.assert_called_with(
            identity_fakes.consumer_id,
        )
        self.assertIsNone(result)


class TestConsumerList(TestOAuth1):

    def setUp(self):
        super(TestConsumerList, self).setUp()

        self.consumers_mock.get.return_value = fakes.FakeResource(
            None,
            copy.deepcopy(identity_fakes.OAUTH_CONSUMER),
            loaded=True,
        )
        self.consumers_mock.list.return_value = [
            fakes.FakeResource(
                None,
                copy.deepcopy(identity_fakes.OAUTH_CONSUMER),
                loaded=True,
            ),
        ]

        # Get the command object to test
        self.cmd = consumer.ListConsumer(self.app, None)

    def test_consumer_list(self):
        arglist = []
        verifylist = []
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # In base command class Lister in cliff, abstract method take_action()
        # returns a tuple containing the column names and an iterable
        # containing the data to be listed.
        columns, data = self.cmd.take_action(parsed_args)
        self.consumers_mock.list.assert_called_with()

        collist = ('ID', 'Description')
        self.assertEqual(collist, columns)
        datalist = ((
            identity_fakes.consumer_id,
            identity_fakes.consumer_description,
        ), )
        self.assertEqual(datalist, tuple(data))


class TestConsumerSet(TestOAuth1):

    def setUp(self):
        super(TestConsumerSet, self).setUp()

        self.consumers_mock.get.return_value = fakes.FakeResource(
            None,
            copy.deepcopy(identity_fakes.OAUTH_CONSUMER),
            loaded=True,
        )

        consumer_updated = copy.deepcopy(identity_fakes.OAUTH_CONSUMER)
        consumer_updated['description'] = "consumer new description"
        self.consumers_mock.update.return_value = fakes.FakeResource(
            None,
            consumer_updated,
            loaded=True,
        )

        self.cmd = consumer.SetConsumer(self.app, None)

    def test_consumer_update(self):
        new_description = "consumer new description"

        arglist = [
            '--description', new_description,
            identity_fakes.consumer_id,
        ]
        verifylist = [
            ('description', new_description),
            ('consumer', identity_fakes.consumer_id),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        result = self.cmd.take_action(parsed_args)

        kwargs = {'description': new_description}
        self.consumers_mock.update.assert_called_with(
            identity_fakes.consumer_id,
            **kwargs
        )
        self.assertIsNone(result)


class TestConsumerShow(TestOAuth1):

    def setUp(self):
        super(TestConsumerShow, self).setUp()

        consumer_no_secret = copy.deepcopy(identity_fakes.OAUTH_CONSUMER)
        del consumer_no_secret['secret']
        self.consumers_mock.get.return_value = fakes.FakeResource(
            None,
            consumer_no_secret,
            loaded=True,
        )

        # Get the command object to test
        self.cmd = consumer.ShowConsumer(self.app, None)

    def test_consumer_show(self):
        arglist = [
            identity_fakes.consumer_id,
        ]
        verifylist = [
            ('consumer', identity_fakes.consumer_id),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        columns, data = self.cmd.take_action(parsed_args)

        self.consumers_mock.get.assert_called_with(
            identity_fakes.consumer_id,
        )

        collist = ('description', 'id')
        self.assertEqual(collist, columns)
        datalist = (
            identity_fakes.consumer_description,
            identity_fakes.consumer_id,
        )
        self.assertEqual(datalist, data)



# Copyright (c) 2016, Intel Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from openstackclient.tests.functional.network.v2 import common


class NetworkQosRuleTypeTests(common.NetworkTests):
    """Functional tests for Network QoS rule type. """

    AVAILABLE_RULE_TYPES = ['dscp_marking',
                            'bandwidth_limit']
    # NOTE(ralonsoh): this list was updated in Yoga (February 2022)
    ALL_AVAILABLE_RULE_TYPES = ['dscp_marking',
                                'bandwidth_limit',
                                'minimum_bandwidth',
                                'packet_rate_limit',
                                'minimum_packet_rate',
                                ]

    def setUp(self):
        super(NetworkQosRuleTypeTests, self).setUp()
        # Nothing in this class works with Nova Network
        if not self.haz_network:
            self.skipTest("No Network service present")

    def test_qos_rule_type_list(self):
        cmd_output = self.openstack(
            'network qos rule type list -f json',
            parse_output=True,
        )
        for rule_type in self.AVAILABLE_RULE_TYPES:
            self.assertIn(rule_type, [x['Type'] for x in cmd_output])

    def test_qos_rule_type_list_all_supported(self):
        if not self.is_extension_enabled('qos-rule-type-filter'):
            self.skipTest('No "qos-rule-type-filter" extension present')

        cmd_output = self.openstack(
            'network qos rule type list --all-supported -f json',
            parse_output=True
        )
        for rule_type in self.AVAILABLE_RULE_TYPES:
            self.assertIn(rule_type, [x['Type'] for x in cmd_output])

    def test_qos_rule_type_list_all_rules(self):
        if not self.is_extension_enabled('qos-rule-type-filter'):
            self.skipTest('No "qos-rule-type-filter" extension present')

        cmd_output = self.openstack(
            'network qos rule type list --all-rules -f json',
            parse_output=True
        )
        for rule_type in self.ALL_AVAILABLE_RULE_TYPES:
            self.assertIn(rule_type, [x['Type'] for x in cmd_output])

    def test_qos_rule_type_details(self):
        for rule_type in self.AVAILABLE_RULE_TYPES:
            cmd_output = self.openstack(
                'network qos rule type show %s -f json' % rule_type,
                parse_output=True,
            )
            self.assertEqual(rule_type, cmd_output['rule_type_name'])
            self.assertIn("drivers", cmd_output.keys())



#   Copyright 2012 OpenStack Foundation
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#

import logging

from osc_lib.command import command
from osc_lib import utils

from openstackclient.i18n import _
from openstackclient.identity import common as identity_common


LOG = logging.getLogger(__name__)


class ProjectPurge(command.Command):
    _description = _("Clean resources associated with a project")

    def get_parser(self, prog_name):
        parser = super(ProjectPurge, self).get_parser(prog_name)
        parser.add_argument(
            '--dry-run',
            action='store_true',
            help=_("List a project's resources"),
        )
        parser.add_argument(
            '--keep-project',
            action='store_true',
            help=_("Clean project resources, but don't delete the project"),
        )
        project_group = parser.add_mutually_exclusive_group(required=True)
        project_group.add_argument(
            '--auth-project',
            action='store_true',
            help=_('Delete resources of the project used to authenticate'),
        )
        project_group.add_argument(
            '--project',
            metavar='<project>',
            help=_('Project to clean (name or ID)'),
        )
        identity_common.add_project_domain_option_to_parser(parser)
        return parser

    def take_action(self, parsed_args):
        identity_client = self.app.client_manager.identity

        if parsed_args.auth_project:
            project_id = self.app.client_manager.auth_ref.project_id
        elif parsed_args.project:
            try:
                project_id = identity_common.find_project(
                    identity_client,
                    parsed_args.project,
                    parsed_args.project_domain,
                ).id
            except AttributeError:  # using v2 auth and supplying a domain
                project_id = utils.find_resource(
                    identity_client.tenants,
                    parsed_args.project,
                ).id

        # delete all non-identity resources
        self.delete_resources(parsed_args.dry_run, project_id)

        # clean up the project
        if not parsed_args.keep_project:
            LOG.warning(_('Deleting project: %s'), project_id)
            if not parsed_args.dry_run:
                identity_client.projects.delete(project_id)

    def delete_resources(self, dry_run, project_id):
        # servers
        try:
            compute_client = self.app.client_manager.compute
            search_opts = {'tenant_id': project_id, 'all_tenants': True}
            data = compute_client.servers.list(search_opts=search_opts)
            self.delete_objects(
                compute_client.servers.delete, data, 'server', dry_run)
        except Exception:
            pass

        # images
        try:
            image_client = self.app.client_manager.image
            api_version = int(image_client.version)
            if api_version == 1:
                data = image_client.images.list(owner=project_id)
            elif api_version == 2:
                kwargs = {'filters': {'owner': project_id}}
                data = image_client.images.list(**kwargs)
            else:
                raise NotImplementedError
            self.delete_objects(
                image_client.images.delete, data, 'image', dry_run)
        except Exception:
            pass

        # volumes, snapshots, backups
        volume_client = self.app.client_manager.volume
        search_opts = {'project_id': project_id, 'all_tenants': True}
        try:
            data = volume_client.volume_snapshots.list(search_opts=search_opts)
            self.delete_objects(
                self.delete_one_volume_snapshot,
                data,
                'volume snapshot',
                dry_run)
        except Exception:
            pass
        try:
            data = volume_client.backups.list(search_opts=search_opts)
            self.delete_objects(
                self.delete_one_volume_backup,
                data,
                'volume backup',
                dry_run)
        except Exception:
            pass
        try:
            data = volume_client.volumes.list(search_opts=search_opts)
            self.delete_objects(
                volume_client.volumes.force_delete, data, 'volume', dry_run)
        except Exception:
            pass

    def delete_objects(self, func_delete, data, resource, dry_run):
        result = 0
        for i in data:
            LOG.warning(_('Deleting %(resource)s : %(id)s') %
                        {'resource': resource, 'id': i.id})
            if not dry_run:
                try:
                    func_delete(i.id)
                except Exception as e:
                    result += 1
                    LOG.error(_("Failed to delete %(resource)s with "
                                "ID '%(id)s': %(e)s")
                              % {'resource': resource, 'id': i.id, 'e': e})
        if result > 0:
            total = len(data)
            msg = (_("%(result)s of %(total)s %(resource)ss failed "
                   "to delete.") %
                   {'result': result,
                    'total': total,
                    'resource': resource})
            LOG.error(msg)

    def delete_one_volume_snapshot(self, snapshot_id):
        volume_client = self.app.client_manager.volume
        try:
            volume_client.volume_snapshots.delete(snapshot_id)
        except Exception:
            # Only volume v2 support deleting by force
            volume_client.volume_snapshots.delete(snapshot_id, force=True)

    def delete_one_volume_backup(self, backup_id):
        volume_client = self.app.client_manager.volume
        try:
            volume_client.backups.delete(backup_id)
        except Exception:
            # Only volume v2 support deleting by force
            volume_client.backups.delete(backup_id, force=True)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import os

from tempest.lib.common.utils import data_utils

from openstackclient.tests.functional.identity.v3 import common

SYSTEM_CLOUD = os.environ.get('OS_SYSTEM_CLOUD', 'devstack-system-admin')


class RegisteredLimitTestCase(common.IdentityTests):

    def test_registered_limit_create_with_service_name(self):
        self._create_dummy_registered_limit()

    def test_registered_limit_create_with_service_id(self):
        service_name = self._create_dummy_service()
        raw_output = self.openstack(
            'service show'
            ' %(service_name)s' % {'service_name': service_name}
        )
        service_items = self.parse_show(raw_output)
        service_id = self._extract_value_from_items('id', service_items)

        raw_output = self.openstack(
            'registered limit create'
            ' --service %(service_id)s'
            ' --default-limit %(default_limit)s'
            ' %(resource_name)s' % {
                'service_id': service_id,
                'default_limit': 10,
                'resource_name': 'cores'
            },
            cloud=SYSTEM_CLOUD
        )
        items = self.parse_show(raw_output)
        registered_limit_id = self._extract_value_from_items('id', items)
        self.addCleanup(
            self.openstack,
            'registered limit delete'
            ' %(registered_limit_id)s' % {
                'registered_limit_id': registered_limit_id
            },
            cloud=SYSTEM_CLOUD
        )

        self.assert_show_fields(items, self.REGISTERED_LIMIT_FIELDS)

    def test_registered_limit_create_with_options(self):
        service_name = self._create_dummy_service()
        region_id = self._create_dummy_region()
        params = {
            'service_name': service_name,
            'resource_name': 'cores',
            'default_limit': 10,
            'description': 'default limit for cores',
            'region_id': region_id
        }

        raw_output = self.openstack(
            'registered limit create'
            ' --description \'%(description)s\''
            ' --region %(region_id)s'
            ' --service %(service_name)s'
            ' --default-limit %(default_limit)s'
            ' %(resource_name)s' % params,
            cloud=SYSTEM_CLOUD
        )
        items = self.parse_show(raw_output)
        registered_limit_id = self._extract_value_from_items('id', items)
        self.addCleanup(
            self.openstack,
            'registered limit delete %(registered_limit_id)s' % {
                'registered_limit_id': registered_limit_id
            },
            cloud=SYSTEM_CLOUD
        )

        self.assert_show_fields(items, self.REGISTERED_LIMIT_FIELDS)

    def test_registered_limit_show(self):
        registered_limit_id = self._create_dummy_registered_limit()
        raw_output = self.openstack(
            'registered limit show %(registered_limit_id)s' % {
                'registered_limit_id': registered_limit_id
            }
        )
        items = self.parse_show(raw_output)
        self.assert_show_fields(items, self.REGISTERED_LIMIT_FIELDS)

    def test_registered_limit_set_region_id(self):
        region_id = self._create_dummy_region()
        registered_limit_id = self._create_dummy_registered_limit()

        params = {
            'registered_limit_id': registered_limit_id,
            'region_id': region_id
        }
        raw_output = self.openstack(
            'registered limit set'
            ' %(registered_limit_id)s'
            ' --region %(region_id)s' % params,
            cloud=SYSTEM_CLOUD
        )
        items = self.parse_show(raw_output)
        self.assert_show_fields(items, self.REGISTERED_LIMIT_FIELDS)

    def test_registered_limit_set_description(self):
        registered_limit_id = self._create_dummy_registered_limit()
        params = {
            'registered_limit_id': registered_limit_id,
            'description': 'updated description'
        }
        raw_output = self.openstack(
            'registered limit set'
            ' %(registered_limit_id)s'
            ' --description \'%(description)s\'' % params,
            cloud=SYSTEM_CLOUD
        )
        items = self.parse_show(raw_output)
        self.assert_show_fields(items, self.REGISTERED_LIMIT_FIELDS)

    def test_registered_limit_set_service(self):
        registered_limit_id = self._create_dummy_registered_limit()
        service_name = self._create_dummy_service()
        params = {
            'registered_limit_id': registered_limit_id,
            'service': service_name
        }
        raw_output = self.openstack(
            'registered limit set'
            ' %(registered_limit_id)s'
            ' --service %(service)s' % params,
            cloud=SYSTEM_CLOUD
        )
        items = self.parse_show(raw_output)
        self.assert_show_fields(items, self.REGISTERED_LIMIT_FIELDS)

    def test_registered_limit_set_default_limit(self):
        registered_limit_id = self._create_dummy_registered_limit()
        params = {
            'registered_limit_id': registered_limit_id,
            'default_limit': 20
        }
        raw_output = self.openstack(
            'registered limit set'
            ' %(registered_limit_id)s'
            ' --default-limit %(default_limit)s' % params,
            cloud=SYSTEM_CLOUD
        )
        items = self.parse_show(raw_output)
        self.assert_show_fields(items, self.REGISTERED_LIMIT_FIELDS)

    def test_registered_limit_set_resource_name(self):
        registered_limit_id = self._create_dummy_registered_limit()
        resource_name = data_utils.rand_name('resource_name')
        params = {
            'registered_limit_id': registered_limit_id,
            'resource_name': resource_name
        }
        raw_output = self.openstack(
            'registered limit set'
            ' %(registered_limit_id)s'
            ' --resource-name %(resource_name)s' % params,
            cloud=SYSTEM_CLOUD
        )
        items = self.parse_show(raw_output)
        self.assert_show_fields(items, self.REGISTERED_LIMIT_FIELDS)

    def test_registered_limit_list(self):
        self._create_dummy_registered_limit()
        raw_output = self.openstack('registered limit list')
        items = self.parse_listing(raw_output)
        self.assert_table_structure(items, self.REGISTERED_LIMIT_LIST_HEADERS)

    def test_registered_limit_delete(self):
        registered_limit_id = self._create_dummy_registered_limit(
            add_clean_up=False
        )
        raw_output = self.openstack(
            'registered limit delete'
            ' %(registered_limit_id)s' % {
                'registered_limit_id': registered_limit_id
            },
            cloud=SYSTEM_CLOUD
        )
        self.assertEqual(0, len(raw_output))



#   Copyright 2012-2013 OpenStack Foundation
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#

"""Group action implementations"""

import logging

from keystoneauth1 import exceptions as ks_exc
from osc_lib.command import command
from osc_lib import exceptions
from osc_lib import utils

from openstackclient.i18n import _
from openstackclient.identity import common


LOG = logging.getLogger(__name__)


class AddUserToGroup(command.Command):
    _description = _("Add user to group")

    def get_parser(self, prog_name):
        parser = super(AddUserToGroup, self).get_parser(prog_name)
        parser.add_argument(
            'group',
            metavar='<group>',
            help=_('Group to contain <user> (name or ID)'),
        )
        parser.add_argument(
            'user',
            metavar='<user>',
            nargs='+',
            help=_('User(s) to add to <group> (name or ID) '
                   '(repeat option to add multiple users)'),
        )
        common.add_group_domain_option_to_parser(parser)
        common.add_user_domain_option_to_parser(parser)
        return parser

    def take_action(self, parsed_args):
        identity_client = self.app.client_manager.identity

        group_id = common.find_group(identity_client,
                                     parsed_args.group,
                                     parsed_args.group_domain).id

        result = 0
        for i in parsed_args.user:
            try:
                user_id = common.find_user(identity_client,
                                           i,
                                           parsed_args.user_domain).id
                identity_client.users.add_to_group(user_id, group_id)
            except Exception as e:
                result += 1
                msg = _("%(user)s not added to group %(group)s: %(e)s") % {
                    'user': i,
                    'group': parsed_args.group,
                    'e': e,
                }
                LOG.error(msg)
        if result > 0:
            total = len(parsed_args.user)
            msg = (_("%(result)s of %(total)s users not added to group "
                   "%(group)s.")) % {
                'result': result,
                'total': total,
                'group': parsed_args.group,
            }
            raise exceptions.CommandError(msg)


class CheckUserInGroup(command.Command):
    _description = _("Check user membership in group")

    def get_parser(self, prog_name):
        parser = super(CheckUserInGroup, self).get_parser(prog_name)
        parser.add_argument(
            'group',
            metavar='<group>',
            help=_('Group to check (name or ID)'),
        )
        parser.add_argument(
            'user',
            metavar='<user>',
            help=_('User to check (name or ID)'),
        )
        common.add_group_domain_option_to_parser(parser)
        common.add_user_domain_option_to_parser(parser)
        return parser

    def take_action(self, parsed_args):
        identity_client = self.app.client_manager.identity

        user_id = common.find_user(identity_client,
                                   parsed_args.user,
                                   parsed_args.user_domain).id
        group_id = common.find_group(identity_client,
                                     parsed_args.group,
                                     parsed_args.group_domain).id

        try:
            identity_client.users.check_in_group(user_id, group_id)
        except ks_exc.http.HTTPClientError as e:
            if e.http_status == 403 or e.http_status == 404:
                msg = _("%(user)s not in group %(group)s\n") % {
                    'user': parsed_args.user,
                    'group': parsed_args.group,
                }
                self.app.stderr.write(msg)
            else:
                raise e
        else:
            msg = _("%(user)s in group %(group)s\n") % {
                'user': parsed_args.user,
                'group': parsed_args.group,
            }
            self.app.stdout.write(msg)


class CreateGroup(command.ShowOne):
    _description = _("Create new group")

    def get_parser(self, prog_name):
        parser = super(CreateGroup, self).get_parser(prog_name)
        parser.add_argument(
            'name',
            metavar='<group-name>',
            help=_('New group name'),
        )
        parser.add_argument(
            '--domain',
            metavar='<domain>',
            help=_('Domain to contain new group (name or ID)'),
        )
        parser.add_argument(
            '--description',
            metavar='<description>',
            help=_('New group description'),
        )
        parser.add_argument(
            '--or-show',
            action='store_true',
            help=_('Return existing group'),
        )
        return parser

    def take_action(self, parsed_args):
        identity_client = self.app.client_manager.identity

        domain = None
        if parsed_args.domain:
            domain = common.find_domain(identity_client,
                                        parsed_args.domain).id

        try:
            group = identity_client.groups.create(
                name=parsed_args.name,
                domain=domain,
                description=parsed_args.description)
        except ks_exc.Conflict:
            if parsed_args.or_show:
                group = utils.find_resource(identity_client.groups,
                                            parsed_args.name,
                                            domain_id=domain)
                LOG.info(_('Returning existing group %s'), group.name)
            else:
                raise

        group._info.pop('links')
        return zip(*sorted(group._info.items()))


class DeleteGroup(command.Command):
    _description = _("Delete group(s)")

    def get_parser(self, prog_name):
        parser = super(DeleteGroup, self).get_parser(prog_name)
        parser.add_argument(
            'groups',
            metavar='<group>',
            nargs="+",
            help=_('Group(s) to delete (name or ID)'),
        )
        parser.add_argument(
            '--domain',
            metavar='<domain>',
            help=_('Domain containing group(s) (name or ID)'),
        )
        return parser

    def take_action(self, parsed_args):
        identity_client = self.app.client_manager.identity

        errors = 0
        for group in parsed_args.groups:
            try:
                group_obj = common.find_group(identity_client,
                                              group,
                                              parsed_args.domain)
                identity_client.groups.delete(group_obj.id)
            except Exception as e:
                errors += 1
                LOG.error(_("Failed to delete group with "
                          "name or ID '%(group)s': %(e)s"),
                          {'group': group, 'e': e})

        if errors > 0:
            total = len(parsed_args.groups)
            msg = (_("%(errors)s of %(total)s groups failed "
                   "to delete.") % {'errors': errors, 'total': total})
            raise exceptions.CommandError(msg)


class ListGroup(command.Lister):
    _description = _("List groups")

    def get_parser(self, prog_name):
        parser = super(ListGroup, self).get_parser(prog_name)
        parser.add_argument(
            '--domain',
            metavar='<domain>',
            help=_('Filter group list by <domain> (name or ID)'),
        )
        parser.add_argument(
            '--user',
            metavar='<user>',
            help=_('Filter group list by <user> (name or ID)'),
        )
        common.add_user_domain_option_to_parser(parser)
        parser.add_argument(
            '--long',
            action='store_true',
            default=False,
            help=_('List additional fields in output'),
        )
        return parser

    def take_action(self, parsed_args):
        identity_client = self.app.client_manager.identity

        domain = None
        if parsed_args.domain:
            domain = common.find_domain(identity_client,
                                        parsed_args.domain).id

        if parsed_args.user:
            user = common.find_user(
                identity_client,
                parsed_args.user,
                parsed_args.user_domain,
            ).id
        else:
            user = None

        # List groups
        if parsed_args.long:
            columns = ('ID', 'Name', 'Domain ID', 'Description')
        else:
            columns = ('ID', 'Name')
        data = identity_client.groups.list(
            domain=domain,
            user=user,
        )

        return (
            columns,
            (utils.get_item_properties(
                s, columns,
                formatters={},
            ) for s in data)
        )


class RemoveUserFromGroup(command.Command):
    _description = _("Remove user from group")

    def get_parser(self, prog_name):
        parser = super(RemoveUserFromGroup, self).get_parser(prog_name)
        parser.add_argument(
            'group',
            metavar='<group>',
            help=_('Group containing <user> (name or ID)'),
        )
        parser.add_argument(
            'user',
            metavar='<user>',
            nargs='+',
            help=_('User(s) to remove from <group> (name or ID) '
                   '(repeat option to remove multiple users)'),
        )
        common.add_group_domain_option_to_parser(parser)
        common.add_user_domain_option_to_parser(parser)
        return parser

    def take_action(self, parsed_args):
        identity_client = self.app.client_manager.identity

        group_id = common.find_group(identity_client,
                                     parsed_args.group,
                                     parsed_args.group_domain).id

        result = 0
        for i in parsed_args.user:
            try:
                user_id = common.find_user(identity_client,
                                           i,
                                           parsed_args.user_domain).id
                identity_client.users.remove_from_group(user_id, group_id)
            except Exception as e:
                result += 1
                msg = _("%(user)s not removed from group %(group)s: %(e)s") % {
                    'user': i,
                    'group': parsed_args.group,
                    'e': e,
                }
                LOG.error(msg)
        if result > 0:
            total = len(parsed_args.user)
            msg = (_("%(result)s of %(total)s users not removed from group "
                   "%(group)s.")) % {
                'result': result,
                'total': total,
                'group': parsed_args.group,
            }
            raise exceptions.CommandError(msg)


class SetGroup(command.Command):
    _description = _("Set group properties")

    def get_parser(self, prog_name):
        parser = super(SetGroup, self).get_parser(prog_name)
        parser.add_argument(
            'group',
            metavar='<group>',
            help=_('Group to modify (name or ID)'),
        )
        parser.add_argument(
            '--domain',
            metavar='<domain>',
            help=_('Domain containing <group> (name or ID)'),
        )
        parser.add_argument(
            '--name',
            metavar='<name>',
            help=_('New group name'),
        )
        parser.add_argument(
            '--description',
            metavar='<description>',
            help=_('New group description'),
        )
        return parser

    def take_action(self, parsed_args):
        identity_client = self.app.client_manager.identity
        group = common.find_group(identity_client, parsed_args.group,
                                  parsed_args.domain)
        kwargs = {}
        if parsed_args.name:
            kwargs['name'] = parsed_args.name
        if parsed_args.description:
            kwargs['description'] = parsed_args.description

        identity_client.groups.update(group.id, **kwargs)


class ShowGroup(command.ShowOne):
    _description = _("Display group details")

    def get_parser(self, prog_name):
        parser = super(ShowGroup, self).get_parser(prog_name)
        parser.add_argument(
            'group',
            metavar='<group>',
            help=_('Group to display (name or ID)'),
        )
        parser.add_argument(
            '--domain',
            metavar='<domain>',
            help=_('Domain containing <group> (name or ID)'),
        )
        return parser

    def take_action(self, parsed_args):
        identity_client = self.app.client_manager.identity

        group = common.find_group(identity_client,
                                  parsed_args.group,
                                  domain_name_or_id=parsed_args.domain)

        group._info.pop('links')
        return zip(*sorted(group._info.items()))



#   Copyright 2012-2013 OpenStack Foundation
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#

"""Identity v3 Endpoint action implementations"""

import logging

from osc_lib.command import command
from osc_lib import exceptions
from osc_lib import utils

from openstackclient.i18n import _
from openstackclient.identity import common


LOG = logging.getLogger(__name__)


def get_service_name(service):
    if hasattr(service, 'name'):
        return service.name
    else:
        return ''


class AddProjectToEndpoint(command.Command):
    _description = _("Associate a project to an endpoint")

    def get_parser(self, prog_name):
        parser = super(
            AddProjectToEndpoint, self).get_parser(prog_name)
        parser.add_argument(
            'endpoint',
            metavar='<endpoint>',
            help=_('Endpoint to associate with '
                   'specified project (name or ID)'),
        )
        parser.add_argument(
            'project',
            metavar='<project>',
            help=_('Project to associate with '
                   'specified endpoint name or ID)'),
        )
        common.add_project_domain_option_to_parser(parser)
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.identity

        endpoint = utils.find_resource(client.endpoints,
                                       parsed_args.endpoint)

        project = common.find_project(client,
                                      parsed_args.project,
                                      parsed_args.project_domain)

        client.endpoint_filter.add_endpoint_to_project(
            project=project.id,
            endpoint=endpoint.id)


class CreateEndpoint(command.ShowOne):
    _description = _("Create new endpoint")

    def get_parser(self, prog_name):
        parser = super(CreateEndpoint, self).get_parser(prog_name)
        parser.add_argument(
            'service',
            metavar='<service>',
            help=_('Service to be associated with new endpoint (name or ID)'),
        )
        parser.add_argument(
            'interface',
            metavar='<interface>',
            choices=['admin', 'public', 'internal'],
            help=_('New endpoint interface type (admin, public or internal)'),
        )
        parser.add_argument(
            'url',
            metavar='<url>',
            help=_('New endpoint URL'),
        )
        parser.add_argument(
            '--region',
            metavar='<region-id>',
            help=_('New endpoint region ID'),
        )
        enable_group = parser.add_mutually_exclusive_group()
        enable_group.add_argument(
            '--enable',
            dest='enabled',
            action='store_true',
            default=True,
            help=_('Enable endpoint (default)'),
        )
        enable_group.add_argument(
            '--disable',
            dest='enabled',
            action='store_false',
            help=_('Disable endpoint'),
        )
        return parser

    def take_action(self, parsed_args):
        identity_client = self.app.client_manager.identity
        service = common.find_service(identity_client, parsed_args.service)

        endpoint = identity_client.endpoints.create(
            service=service.id,
            url=parsed_args.url,
            interface=parsed_args.interface,
            region=parsed_args.region,
            enabled=parsed_args.enabled
        )

        info = {}
        endpoint._info.pop('links')
        info.update(endpoint._info)
        info['service_name'] = get_service_name(service)
        info['service_type'] = service.type
        return zip(*sorted(info.items()))


class DeleteEndpoint(command.Command):
    _description = _("Delete endpoint(s)")

    def get_parser(self, prog_name):
        parser = super(DeleteEndpoint, self).get_parser(prog_name)
        parser.add_argument(
            'endpoint',
            metavar='<endpoint-id>',
            nargs='+',
            help=_('Endpoint(s) to delete (ID only)'),
        )
        return parser

    def take_action(self, parsed_args):
        identity_client = self.app.client_manager.identity
        result = 0
        for i in parsed_args.endpoint:
            try:
                endpoint_id = utils.find_resource(
                    identity_client.endpoints, i).id
                identity_client.endpoints.delete(endpoint_id)
            except Exception as e:
                result += 1
                LOG.error(_("Failed to delete endpoint with "
                          "ID '%(endpoint)s': %(e)s"), {'endpoint': i, 'e': e})

        if result > 0:
            total = len(parsed_args.endpoint)
            msg = (_("%(result)s of %(total)s endpoints failed "
                   "to delete.") % {'result': result, 'total': total})
            raise exceptions.CommandError(msg)


class ListEndpoint(command.Lister):
    _description = _("List endpoints")

    def get_parser(self, prog_name):
        parser = super(ListEndpoint, self).get_parser(prog_name)
        parser.add_argument(
            '--service',
            metavar='<service>',
            help=_('Filter by service (type, name or ID)'),
        )
        parser.add_argument(
            '--interface',
            metavar='<interface>',
            choices=['admin', 'public', 'internal'],
            help=_('Filter by interface type (admin, public or internal)'),
        )
        parser.add_argument(
            '--region',
            metavar='<region-id>',
            help=_('Filter by region ID'),
        )
        list_group = parser.add_mutually_exclusive_group()
        list_group.add_argument(
            '--endpoint',
            metavar='<endpoint-group>',
            help=_('Endpoint to list filters'),
        )
        list_group.add_argument(
            '--project',
            metavar='<project>',
            help=_('Project to list filters (name or ID)'),
        )
        common.add_project_domain_option_to_parser(parser)
        return parser

    def take_action(self, parsed_args):
        identity_client = self.app.client_manager.identity

        endpoint = None
        if parsed_args.endpoint:
            endpoint = utils.find_resource(identity_client.endpoints,
                                           parsed_args.endpoint)
        project = None
        if parsed_args.project:
            project = common.find_project(identity_client,
                                          parsed_args.project,
                                          parsed_args.project_domain)

        if endpoint:
            columns = ('ID', 'Name')
            data = (
                identity_client.endpoint_filter
                .list_projects_for_endpoint(endpoint=endpoint.id)
            )
        else:
            columns = ('ID', 'Region', 'Service Name', 'Service Type',
                       'Enabled', 'Interface', 'URL')
            kwargs = {}
            if parsed_args.service:
                service = common.find_service(identity_client,
                                              parsed_args.service)
                kwargs['service'] = service.id
            if parsed_args.interface:
                kwargs['interface'] = parsed_args.interface
            if parsed_args.region:
                kwargs['region'] = parsed_args.region

            if project:
                data = (
                    identity_client.endpoint_filter
                    .list_endpoints_for_project(project=project.id)
                )
            else:
                data = identity_client.endpoints.list(**kwargs)

            service_list = identity_client.services.list()

            for ep in data:
                service = common.find_service_in_list(service_list,
                                                      ep.service_id)
                ep.service_name = get_service_name(service)
                ep.service_type = service.type

        return (columns,
                (utils.get_item_properties(
                    s, columns,
                    formatters={},
                ) for s in data))


class RemoveProjectFromEndpoint(command.Command):
    _description = _("Dissociate a project from an endpoint")

    def get_parser(self, prog_name):
        parser = super(
            RemoveProjectFromEndpoint, self).get_parser(prog_name)
        parser.add_argument(
            'endpoint',
            metavar='<endpoint>',
            help=_('Endpoint to dissociate from '
                   'specified project (name or ID)'),
        )
        parser.add_argument(
            'project',
            metavar='<project>',
            help=_('Project to dissociate from '
                   'specified endpoint name or ID)'),
        )
        common.add_project_domain_option_to_parser(parser)
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.identity

        endpoint = utils.find_resource(client.endpoints,
                                       parsed_args.endpoint)

        project = common.find_project(client,
                                      parsed_args.project,
                                      parsed_args.project_domain)

        client.endpoint_filter.delete_endpoint_from_project(
            project=project.id,
            endpoint=endpoint.id)


class SetEndpoint(command.Command):
    _description = _("Set endpoint properties")

    def get_parser(self, prog_name):
        parser = super(SetEndpoint, self).get_parser(prog_name)
        parser.add_argument(
            'endpoint',
            metavar='<endpoint-id>',
            help=_('Endpoint to modify (ID only)'),
        )
        parser.add_argument(
            '--region',
            metavar='<region-id>',
            help=_('New endpoint region ID'),
        )
        parser.add_argument(
            '--interface',
            metavar='<interface>',
            choices=['admin', 'public', 'internal'],
            help=_('New endpoint interface type (admin, public or internal)'),
        )
        parser.add_argument(
            '--url',
            metavar='<url>',
            help=_('New endpoint URL'),
        )
        parser.add_argument(
            '--service',
            metavar='<service>',
            help=_('New endpoint service (name or ID)'),
        )
        enable_group = parser.add_mutually_exclusive_group()
        enable_group.add_argument(
            '--enable',
            dest='enabled',
            action='store_true',
            help=_('Enable endpoint'),
        )
        enable_group.add_argument(
            '--disable',
            dest='disabled',
            action='store_true',
            help=_('Disable endpoint'),
        )
        return parser

    def take_action(self, parsed_args):
        identity_client = self.app.client_manager.identity
        endpoint = utils.find_resource(identity_client.endpoints,
                                       parsed_args.endpoint)

        service_id = None
        if parsed_args.service:
            service = common.find_service(identity_client, parsed_args.service)
            service_id = service.id
        enabled = None
        if parsed_args.enabled:
            enabled = True
        if parsed_args.disabled:
            enabled = False

        identity_client.endpoints.update(
            endpoint.id,
            service=service_id,
            url=parsed_args.url,
            interface=parsed_args.interface,
            region=parsed_args.region,
            enabled=enabled
        )


class ShowEndpoint(command.ShowOne):
    _description = _("Display endpoint details")

    def get_parser(self, prog_name):
        parser = super(ShowEndpoint, self).get_parser(prog_name)
        parser.add_argument(
            'endpoint',
            metavar='<endpoint>',
            help=_('Endpoint to display (endpoint ID, service ID,'
                   ' service name, service type)'),
        )
        return parser

    def take_action(self, parsed_args):
        identity_client = self.app.client_manager.identity
        endpoint = utils.find_resource(identity_client.endpoints,
                                       parsed_args.endpoint)

        service = common.find_service(identity_client, endpoint.service_id)

        info = {}
        endpoint._info.pop('links')
        info.update(endpoint._info)
        info['service_name'] = get_service_name(service)
        info['service_type'] = service.type
        return zip(*sorted(info.items()))
